"""
Calculation modes
"""
from __future__ import annotations
import typing
__all__ = ['AllAggregate', 'Arithmetic', 'AspenFireSeverity', 'BySizeClass', 'Chamise', 'ChaparralContants', 'ChaparralFuelInputLoadMode', 'ChaparralFuelType', 'Dead', 'DeadAggregate', 'DeadAggregateAndLiveSizeClass', 'DirectFuelLoad', 'DirectMidflame', 'DontUseCrownRatio', 'First', 'FromIgnitionPoint', 'FromPerimeter', 'FuelConstants', 'FuelLifeState', 'FuelLoadFromDepthAndChaparralType', 'Harmonic', 'HundredHour', 'Live', 'LiveAggregate', 'LiveAggregateAndDeadSizeClass', 'LiveHerbaceous', 'LiveWoody', 'Low', 'MaxDeadSizeClasses', 'MaxFuelModels', 'MaxLifeStates', 'MaxLiveSizeClasses', 'MaxParticles', 'MaxSavrSizeClasses', 'MixedBrush', 'Moderate', 'MoistureClassInput', 'MoistureInputMode', 'MoistureScenario', 'NoMethod', 'NotSet', 'NumFuelClasses', 'NumberOfModels', 'OneHour', 'RelativeToNorth', 'RelativeToUpslope', 'Second', 'Sheltered', 'SurfaceFireSpreadDirectionMode', 'TenHour', 'TenMeter', 'TwentyFoot', 'TwoDimensional', 'TwoFuelModelsContants', 'TwoFuelModelsMethod', 'Unsheltered', 'UseCrownRatio', 'UserInput', 'WindAdjustmentFactorCalculationMethod', 'WindAdjustmentFactorShelterMethod', 'WindAndSpreadOrientationMode', 'WindHeightInputMode']
class AspenFireSeverity:
    """
    Members:
    
      Low
    
      Moderate
    """
    Low: typing.ClassVar[AspenFireSeverity]  # value = <AspenFireSeverity.Low: 0>
    Moderate: typing.ClassVar[AspenFireSeverity]  # value = <AspenFireSeverity.Moderate: 1>
    __members__: typing.ClassVar[dict[str, AspenFireSeverity]]  # value = {'Low': <AspenFireSeverity.Low: 0>, 'Moderate': <AspenFireSeverity.Moderate: 1>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class ChaparralContants:
    """
    Members:
    
      NumFuelClasses
    """
    NumFuelClasses: typing.ClassVar[ChaparralContants]  # value = <ChaparralContants.NumFuelClasses: 5>
    __members__: typing.ClassVar[dict[str, ChaparralContants]]  # value = {'NumFuelClasses': <ChaparralContants.NumFuelClasses: 5>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class ChaparralFuelInputLoadMode:
    """
    Members:
    
      DirectFuelLoad
    
      FuelLoadFromDepthAndChaparralType
    """
    DirectFuelLoad: typing.ClassVar[ChaparralFuelInputLoadMode]  # value = <ChaparralFuelInputLoadMode.DirectFuelLoad: 1>
    FuelLoadFromDepthAndChaparralType: typing.ClassVar[ChaparralFuelInputLoadMode]  # value = <ChaparralFuelInputLoadMode.FuelLoadFromDepthAndChaparralType: 2>
    __members__: typing.ClassVar[dict[str, ChaparralFuelInputLoadMode]]  # value = {'DirectFuelLoad': <ChaparralFuelInputLoadMode.DirectFuelLoad: 1>, 'FuelLoadFromDepthAndChaparralType': <ChaparralFuelInputLoadMode.FuelLoadFromDepthAndChaparralType: 2>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class ChaparralFuelType:
    """
    Members:
    
      NotSet
    
      Chamise
    
      MixedBrush
    """
    Chamise: typing.ClassVar[ChaparralFuelType]  # value = <ChaparralFuelType.Chamise: 1>
    MixedBrush: typing.ClassVar[ChaparralFuelType]  # value = <ChaparralFuelType.MixedBrush: 2>
    NotSet: typing.ClassVar[ChaparralFuelType]  # value = <ChaparralFuelType.NotSet: 0>
    __members__: typing.ClassVar[dict[str, ChaparralFuelType]]  # value = {'NotSet': <ChaparralFuelType.NotSet: 0>, 'Chamise': <ChaparralFuelType.Chamise: 1>, 'MixedBrush': <ChaparralFuelType.MixedBrush: 2>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class FuelConstants:
    """
    Members:
    
      MaxLifeStates
    
      MaxLiveSizeClasses
    
      MaxDeadSizeClasses
    
      MaxParticles
    
      MaxSavrSizeClasses
    
      MaxFuelModels
    """
    MaxDeadSizeClasses: typing.ClassVar[FuelConstants]  # value = <FuelConstants.MaxDeadSizeClasses: 4>
    MaxFuelModels: typing.ClassVar[FuelConstants]  # value = <FuelConstants.MaxFuelModels: 256>
    MaxLifeStates: typing.ClassVar[FuelConstants]  # value = <FuelConstants.MaxLifeStates: 2>
    MaxLiveSizeClasses: typing.ClassVar[FuelConstants]  # value = <FuelConstants.MaxLiveSizeClasses: 5>
    MaxParticles: typing.ClassVar[FuelConstants]  # value = <FuelConstants.MaxLiveSizeClasses: 5>
    MaxSavrSizeClasses: typing.ClassVar[FuelConstants]  # value = <FuelConstants.MaxLiveSizeClasses: 5>
    __members__: typing.ClassVar[dict[str, FuelConstants]]  # value = {'MaxLifeStates': <FuelConstants.MaxLifeStates: 2>, 'MaxLiveSizeClasses': <FuelConstants.MaxLiveSizeClasses: 5>, 'MaxDeadSizeClasses': <FuelConstants.MaxDeadSizeClasses: 4>, 'MaxParticles': <FuelConstants.MaxLiveSizeClasses: 5>, 'MaxSavrSizeClasses': <FuelConstants.MaxLiveSizeClasses: 5>, 'MaxFuelModels': <FuelConstants.MaxFuelModels: 256>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class FuelLifeState:
    """
    Members:
    
      Dead
    
      Live
    """
    Dead: typing.ClassVar[FuelLifeState]  # value = <FuelLifeState.Dead: 0>
    Live: typing.ClassVar[FuelLifeState]  # value = <FuelLifeState.Live: 1>
    __members__: typing.ClassVar[dict[str, FuelLifeState]]  # value = {'Dead': <FuelLifeState.Dead: 0>, 'Live': <FuelLifeState.Live: 1>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class MoistureClassInput:
    """
    Members:
    
      OneHour
    
      TenHour
    
      HundredHour
    
      LiveHerbaceous
    
      LiveWoody
    
      DeadAggregate
    
      LiveAggregate
    """
    DeadAggregate: typing.ClassVar[MoistureClassInput]  # value = <MoistureClassInput.DeadAggregate: 5>
    HundredHour: typing.ClassVar[MoistureClassInput]  # value = <MoistureClassInput.HundredHour: 2>
    LiveAggregate: typing.ClassVar[MoistureClassInput]  # value = <MoistureClassInput.LiveAggregate: 6>
    LiveHerbaceous: typing.ClassVar[MoistureClassInput]  # value = <MoistureClassInput.LiveHerbaceous: 3>
    LiveWoody: typing.ClassVar[MoistureClassInput]  # value = <MoistureClassInput.LiveWoody: 4>
    OneHour: typing.ClassVar[MoistureClassInput]  # value = <MoistureClassInput.OneHour: 0>
    TenHour: typing.ClassVar[MoistureClassInput]  # value = <MoistureClassInput.TenHour: 1>
    __members__: typing.ClassVar[dict[str, MoistureClassInput]]  # value = {'OneHour': <MoistureClassInput.OneHour: 0>, 'TenHour': <MoistureClassInput.TenHour: 1>, 'HundredHour': <MoistureClassInput.HundredHour: 2>, 'LiveHerbaceous': <MoistureClassInput.LiveHerbaceous: 3>, 'LiveWoody': <MoistureClassInput.LiveWoody: 4>, 'DeadAggregate': <MoistureClassInput.DeadAggregate: 5>, 'LiveAggregate': <MoistureClassInput.LiveAggregate: 6>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class MoistureInputMode:
    """
    Members:
    
      BySizeClass
    
      AllAggregate
    
      DeadAggregateAndLiveSizeClass
    
      LiveAggregateAndDeadSizeClass
    
      MoistureScenario
    """
    AllAggregate: typing.ClassVar[MoistureInputMode]  # value = <MoistureInputMode.AllAggregate: 1>
    BySizeClass: typing.ClassVar[MoistureInputMode]  # value = <MoistureInputMode.BySizeClass: 0>
    DeadAggregateAndLiveSizeClass: typing.ClassVar[MoistureInputMode]  # value = <MoistureInputMode.DeadAggregateAndLiveSizeClass: 2>
    LiveAggregateAndDeadSizeClass: typing.ClassVar[MoistureInputMode]  # value = <MoistureInputMode.LiveAggregateAndDeadSizeClass: 3>
    MoistureScenario: typing.ClassVar[MoistureInputMode]  # value = <MoistureInputMode.MoistureScenario: 4>
    __members__: typing.ClassVar[dict[str, MoistureInputMode]]  # value = {'BySizeClass': <MoistureInputMode.BySizeClass: 0>, 'AllAggregate': <MoistureInputMode.AllAggregate: 1>, 'DeadAggregateAndLiveSizeClass': <MoistureInputMode.DeadAggregateAndLiveSizeClass: 2>, 'LiveAggregateAndDeadSizeClass': <MoistureInputMode.LiveAggregateAndDeadSizeClass: 3>, 'MoistureScenario': <MoistureInputMode.MoistureScenario: 4>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class SurfaceFireSpreadDirectionMode:
    """
    Members:
    
      FromIgnitionPoint
    
      FromPerimeter
    """
    FromIgnitionPoint: typing.ClassVar[SurfaceFireSpreadDirectionMode]  # value = <SurfaceFireSpreadDirectionMode.FromIgnitionPoint: 0>
    FromPerimeter: typing.ClassVar[SurfaceFireSpreadDirectionMode]  # value = <SurfaceFireSpreadDirectionMode.FromPerimeter: 1>
    __members__: typing.ClassVar[dict[str, SurfaceFireSpreadDirectionMode]]  # value = {'FromIgnitionPoint': <SurfaceFireSpreadDirectionMode.FromIgnitionPoint: 0>, 'FromPerimeter': <SurfaceFireSpreadDirectionMode.FromPerimeter: 1>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class TwoFuelModelsContants:
    """
    Members:
    
      First
    
      Second
    
      NumberOfModels
    """
    First: typing.ClassVar[TwoFuelModelsContants]  # value = <TwoFuelModelsContants.First: 0>
    NumberOfModels: typing.ClassVar[TwoFuelModelsContants]  # value = <TwoFuelModelsContants.NumberOfModels: 2>
    Second: typing.ClassVar[TwoFuelModelsContants]  # value = <TwoFuelModelsContants.Second: 1>
    __members__: typing.ClassVar[dict[str, TwoFuelModelsContants]]  # value = {'First': <TwoFuelModelsContants.First: 0>, 'Second': <TwoFuelModelsContants.Second: 1>, 'NumberOfModels': <TwoFuelModelsContants.NumberOfModels: 2>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class TwoFuelModelsMethod:
    """
    Members:
    
      NoMethod
    
      Arithmetic
    
      Harmonic
    
      TwoDimensional
    """
    Arithmetic: typing.ClassVar[TwoFuelModelsMethod]  # value = <TwoFuelModelsMethod.Arithmetic: 1>
    Harmonic: typing.ClassVar[TwoFuelModelsMethod]  # value = <TwoFuelModelsMethod.Harmonic: 2>
    NoMethod: typing.ClassVar[TwoFuelModelsMethod]  # value = <TwoFuelModelsMethod.NoMethod: 0>
    TwoDimensional: typing.ClassVar[TwoFuelModelsMethod]  # value = <TwoFuelModelsMethod.TwoDimensional: 3>
    __members__: typing.ClassVar[dict[str, TwoFuelModelsMethod]]  # value = {'NoMethod': <TwoFuelModelsMethod.NoMethod: 0>, 'Arithmetic': <TwoFuelModelsMethod.Arithmetic: 1>, 'Harmonic': <TwoFuelModelsMethod.Harmonic: 2>, 'TwoDimensional': <TwoFuelModelsMethod.TwoDimensional: 3>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class WindAdjustmentFactorCalculationMethod:
    """
    Members:
    
      UserInput
    
      UseCrownRatio
    
      DontUseCrownRatio
    """
    DontUseCrownRatio: typing.ClassVar[WindAdjustmentFactorCalculationMethod]  # value = <WindAdjustmentFactorCalculationMethod.DontUseCrownRatio: 2>
    UseCrownRatio: typing.ClassVar[WindAdjustmentFactorCalculationMethod]  # value = <WindAdjustmentFactorCalculationMethod.UseCrownRatio: 1>
    UserInput: typing.ClassVar[WindAdjustmentFactorCalculationMethod]  # value = <WindAdjustmentFactorCalculationMethod.UserInput: 0>
    __members__: typing.ClassVar[dict[str, WindAdjustmentFactorCalculationMethod]]  # value = {'UserInput': <WindAdjustmentFactorCalculationMethod.UserInput: 0>, 'UseCrownRatio': <WindAdjustmentFactorCalculationMethod.UseCrownRatio: 1>, 'DontUseCrownRatio': <WindAdjustmentFactorCalculationMethod.DontUseCrownRatio: 2>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class WindAdjustmentFactorShelterMethod:
    """
    Members:
    
      Unsheltered
    
      Sheltered
    """
    Sheltered: typing.ClassVar[WindAdjustmentFactorShelterMethod]  # value = <WindAdjustmentFactorShelterMethod.Sheltered: 1>
    Unsheltered: typing.ClassVar[WindAdjustmentFactorShelterMethod]  # value = <WindAdjustmentFactorShelterMethod.Unsheltered: 0>
    __members__: typing.ClassVar[dict[str, WindAdjustmentFactorShelterMethod]]  # value = {'Unsheltered': <WindAdjustmentFactorShelterMethod.Unsheltered: 0>, 'Sheltered': <WindAdjustmentFactorShelterMethod.Sheltered: 1>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class WindAndSpreadOrientationMode:
    """
    Members:
    
      RelativeToUpslope
    
      RelativeToNorth
    """
    RelativeToNorth: typing.ClassVar[WindAndSpreadOrientationMode]  # value = <WindAndSpreadOrientationMode.RelativeToNorth: 1>
    RelativeToUpslope: typing.ClassVar[WindAndSpreadOrientationMode]  # value = <WindAndSpreadOrientationMode.RelativeToUpslope: 0>
    __members__: typing.ClassVar[dict[str, WindAndSpreadOrientationMode]]  # value = {'RelativeToUpslope': <WindAndSpreadOrientationMode.RelativeToUpslope: 0>, 'RelativeToNorth': <WindAndSpreadOrientationMode.RelativeToNorth: 1>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
class WindHeightInputMode:
    """
    Members:
    
      DirectMidflame
    
      TwentyFoot
    
      TenMeter
    """
    DirectMidflame: typing.ClassVar[WindHeightInputMode]  # value = <WindHeightInputMode.DirectMidflame: 0>
    TenMeter: typing.ClassVar[WindHeightInputMode]  # value = <WindHeightInputMode.TenMeter: 2>
    TwentyFoot: typing.ClassVar[WindHeightInputMode]  # value = <WindHeightInputMode.TwentyFoot: 1>
    __members__: typing.ClassVar[dict[str, WindHeightInputMode]]  # value = {'DirectMidflame': <WindHeightInputMode.DirectMidflame: 0>, 'TwentyFoot': <WindHeightInputMode.TwentyFoot: 1>, 'TenMeter': <WindHeightInputMode.TenMeter: 2>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
AllAggregate: MoistureInputMode  # value = <MoistureInputMode.AllAggregate: 1>
Arithmetic: TwoFuelModelsMethod  # value = <TwoFuelModelsMethod.Arithmetic: 1>
BySizeClass: MoistureInputMode  # value = <MoistureInputMode.BySizeClass: 0>
Chamise: ChaparralFuelType  # value = <ChaparralFuelType.Chamise: 1>
Dead: FuelLifeState  # value = <FuelLifeState.Dead: 0>
DeadAggregate: MoistureClassInput  # value = <MoistureClassInput.DeadAggregate: 5>
DeadAggregateAndLiveSizeClass: MoistureInputMode  # value = <MoistureInputMode.DeadAggregateAndLiveSizeClass: 2>
DirectFuelLoad: ChaparralFuelInputLoadMode  # value = <ChaparralFuelInputLoadMode.DirectFuelLoad: 1>
DirectMidflame: WindHeightInputMode  # value = <WindHeightInputMode.DirectMidflame: 0>
DontUseCrownRatio: WindAdjustmentFactorCalculationMethod  # value = <WindAdjustmentFactorCalculationMethod.DontUseCrownRatio: 2>
First: TwoFuelModelsContants  # value = <TwoFuelModelsContants.First: 0>
FromIgnitionPoint: SurfaceFireSpreadDirectionMode  # value = <SurfaceFireSpreadDirectionMode.FromIgnitionPoint: 0>
FromPerimeter: SurfaceFireSpreadDirectionMode  # value = <SurfaceFireSpreadDirectionMode.FromPerimeter: 1>
FuelLoadFromDepthAndChaparralType: ChaparralFuelInputLoadMode  # value = <ChaparralFuelInputLoadMode.FuelLoadFromDepthAndChaparralType: 2>
Harmonic: TwoFuelModelsMethod  # value = <TwoFuelModelsMethod.Harmonic: 2>
HundredHour: MoistureClassInput  # value = <MoistureClassInput.HundredHour: 2>
Live: FuelLifeState  # value = <FuelLifeState.Live: 1>
LiveAggregate: MoistureClassInput  # value = <MoistureClassInput.LiveAggregate: 6>
LiveAggregateAndDeadSizeClass: MoistureInputMode  # value = <MoistureInputMode.LiveAggregateAndDeadSizeClass: 3>
LiveHerbaceous: MoistureClassInput  # value = <MoistureClassInput.LiveHerbaceous: 3>
LiveWoody: MoistureClassInput  # value = <MoistureClassInput.LiveWoody: 4>
Low: AspenFireSeverity  # value = <AspenFireSeverity.Low: 0>
MaxDeadSizeClasses: FuelConstants  # value = <FuelConstants.MaxDeadSizeClasses: 4>
MaxFuelModels: FuelConstants  # value = <FuelConstants.MaxFuelModels: 256>
MaxLifeStates: FuelConstants  # value = <FuelConstants.MaxLifeStates: 2>
MaxLiveSizeClasses: FuelConstants  # value = <FuelConstants.MaxLiveSizeClasses: 5>
MaxParticles: FuelConstants  # value = <FuelConstants.MaxLiveSizeClasses: 5>
MaxSavrSizeClasses: FuelConstants  # value = <FuelConstants.MaxLiveSizeClasses: 5>
MixedBrush: ChaparralFuelType  # value = <ChaparralFuelType.MixedBrush: 2>
Moderate: AspenFireSeverity  # value = <AspenFireSeverity.Moderate: 1>
MoistureScenario: MoistureInputMode  # value = <MoistureInputMode.MoistureScenario: 4>
NoMethod: TwoFuelModelsMethod  # value = <TwoFuelModelsMethod.NoMethod: 0>
NotSet: ChaparralFuelType  # value = <ChaparralFuelType.NotSet: 0>
NumFuelClasses: ChaparralContants  # value = <ChaparralContants.NumFuelClasses: 5>
NumberOfModels: TwoFuelModelsContants  # value = <TwoFuelModelsContants.NumberOfModels: 2>
OneHour: MoistureClassInput  # value = <MoistureClassInput.OneHour: 0>
RelativeToNorth: WindAndSpreadOrientationMode  # value = <WindAndSpreadOrientationMode.RelativeToNorth: 1>
RelativeToUpslope: WindAndSpreadOrientationMode  # value = <WindAndSpreadOrientationMode.RelativeToUpslope: 0>
Second: TwoFuelModelsContants  # value = <TwoFuelModelsContants.Second: 1>
Sheltered: WindAdjustmentFactorShelterMethod  # value = <WindAdjustmentFactorShelterMethod.Sheltered: 1>
TenHour: MoistureClassInput  # value = <MoistureClassInput.TenHour: 1>
TenMeter: WindHeightInputMode  # value = <WindHeightInputMode.TenMeter: 2>
TwentyFoot: WindHeightInputMode  # value = <WindHeightInputMode.TwentyFoot: 1>
TwoDimensional: TwoFuelModelsMethod  # value = <TwoFuelModelsMethod.TwoDimensional: 3>
Unsheltered: WindAdjustmentFactorShelterMethod  # value = <WindAdjustmentFactorShelterMethod.Unsheltered: 0>
UseCrownRatio: WindAdjustmentFactorCalculationMethod  # value = <WindAdjustmentFactorCalculationMethod.UseCrownRatio: 1>
UserInput: WindAdjustmentFactorCalculationMethod  # value = <WindAdjustmentFactorCalculationMethod.UserInput: 0>



import pyrothermel
from pyrothermel import behave_core, units, modes

# Initialize model framework
fuel_models = behave_core.FuelModels()
mortality_species_table = behave_core.SpeciesMasterTable()
behave = behave_core.BehaveRun(fuel_models, mortality_species_table)

# Set input values
fuel_model_number = 20
custom_fuel_model = True

# Custom fuel model parameters
code = "Custom"
name = "CustomFuelModel"
fuel_bed_depth = 0.5
length_units = units.LengthUnits.Meters
moisture_of_extinction_dead = 0.3
moisture_units = units.FractionUnits.Fraction
heat_of_combustion_dead = 8000
heat_of_combustion_live = 8000
heat_of_combustion_units = units.HeatOfCombustionUnits.BtusPerPound
fuel_load_one_hour = 0.08
fuel_load_ten_hour = 0.08
fuel_load_hundred_hour = 0.08
fuel_load_live_herbaceous = 0.08
fuel_load_live_woody = 0.08
loading_units = units.LoadingUnits.KilogramsPerSquareMeter
savr_one_hour = 2000
savr_live_herbaceous = 2000
savr_live_woody = 1600
savr_units = units.SurfaceAreaToVolumeUnits.SquareInchesOverCubicInches
is_dynamic = False

if custom_fuel_model:
    fuel_models.setCustomFuelModel(
        fuel_model_number, code, name, fuel_bed_depth, length_units, moisture_of_extinction_dead,
        moisture_units, heat_of_combustion_dead, heat_of_combustion_live, heat_of_combustion_units,
        fuel_load_one_hour, fuel_load_ten_hour, fuel_load_hundred_hour, fuel_load_live_herbaceous,
        fuel_load_live_woody, loading_units, savr_one_hour, savr_live_herbaceous, savr_live_woody,
        savr_units, is_dynamic
    )

# Update inputs in model objects
moisture_one_hour = 0.05
moisture_ten_hour = 0.07
moisture_hundred_hour = 0.09
moisture_live_herbaceous = 0.6
moisture_live_woody = 0.90
wind_speed = 20
wind_speed_units = units.SpeedUnits.MilesPerHour
wind_height_input_mode = modes.WindHeightInputMode.DirectMidflame
wind_direction = 42

slope = 30
aspect = 291
canopy_cover = 0.50  # 50%
canopy_cover_units = units.FractionUnits.Fraction
canopy_height = 6
canopy_height_units = units.LengthUnits.Meters
crown_ratio = 0.50

wind_and_spread_orientation_mode = modes.WindAndSpreadOrientationMode.RelativeToNorth

behave.surface.updateSurfaceInputs(
    fuel_model_number, moisture_one_hour, moisture_ten_hour, moisture_hundred_hour,
    moisture_live_herbaceous, moisture_live_woody, moisture_units, wind_speed, wind_speed_units,
    wind_height_input_mode, wind_direction, wind_and_spread_orientation_mode, slope,
    units.SlopeUnits.Degrees, aspect, canopy_cover, canopy_cover_units, canopy_height,
    canopy_height_units, crown_ratio, units.FractionUnits.Fraction
)

behave.surface.setIsUsingChaparral(False)

# Run model with given inputs
behave.surface.doSurfaceRunInDirectionOfMaxSpread()

# Get results
spread_rate = behave.surface.getSpreadRate(units.SpeedUnits.KilometersPerHour)
print(f"Rate of spread: {spread_rate} km/hr")
flame_length = behave.surface.getFlameLength(units.LengthUnits.Meters)
print(f"Flame length: {flame_length} meters")
direction_of_max_spread = behave.surface.getDirectionOfMaxSpread()
print(f"Direction of maximum spread is {direction_of_max_spread} degrees")

# Run scorch model
fireline_intensity = behave.surface.getFirelineIntensity(units.FirelineIntensityUnits.BtusPerFootPerSecond)
midflame_windspeed = behave.surface.getMidflameWindspeed(units.SpeedUnits.MilesPerHour)
air_temperature = 80.0
scorch_length_units = units.LengthUnits.Meters
observed_scorch_height = behave.mortality.calculateScorchHeight(
    fireline_intensity, units.FirelineIntensityUnits.BtusPerFootPerSecond,
    midflame_windspeed, units.SpeedUnits.MilesPerHour,
    air_temperature, units.TemperatureUnits.Fahrenheit,
    scorch_length_units
)
print(f"Scorch height is {observed_scorch_height} meters")

# Run crown model
moisture_foliar = 1
canopy_base_height = 2.952756
canopy_bulk_density = 0.1
canopy_bulk_density_units = units.DensityUnits.KilogramsPerCubicMeter

behave.crown.setWindAdjustmentFactorCalculationMethod(modes.WindAdjustmentFactorCalculationMethod.UserInput)
behave.crown.setUserProvidedWindAdjustmentFactor(0.4)
behave.crown.updateCrownInputs(
    fuel_model_number, moisture_one_hour, moisture_ten_hour, moisture_hundred_hour,
    moisture_live_herbaceous, moisture_live_woody, moisture_foliar, moisture_units, wind_speed,
    wind_speed_units, wind_height_input_mode, wind_direction, wind_and_spread_orientation_mode, slope,
    units.SlopeUnits.Degrees, aspect, canopy_cover, canopy_cover_units, canopy_height,
    canopy_base_height, canopy_height_units, crown_ratio, units.FractionUnits.Fraction,
    canopy_bulk_density, canopy_bulk_density_units
)

behave.crown.doCrownRunScottAndReinhardt()
crown_fire_spread_rate = behave.crown.getFinalSpreadRate(units.SpeedUnits.KilometersPerHour)
print(f"Crown fire spread rate is {crown_fire_spread_rate} km/hr")
crown_fire_intensity = behave.crown.getFinalFirelineIntensity(units.FirelineIntensityUnits.KilowattsPerMeter)
print(f"Crown fire intensity is {crown_fire_intensity}")
crown_fire_flame_length = behave.crown.getCrownFlameLength(units.LengthUnits.Meters)
print(f"Crown fire flame length is {crown_fire_flame_length}")
transition_ratio = behave.crown.getTransitionRatio()
print(f"Transition ratio is {transition_ratio}")
active_ratio = behave.crown.getActiveRatio()
print(f"Active ratio is {active_ratio}")
fire_type = behave.crown.getFireType()
print(f"Fire type is {fire_type}")


"""
Measurement unit types and conversions
"""
from __future__ import annotations
import typing
__all__ = ['AreaUnits', 'BasalAreaUnits', 'DensityUnits', 'FirelineIntensityUnits', 'FractionUnits', 'HeatOfCombustionUnits', 'HeatPerUnitAreaUnits', 'HeatSinkUnits', 'HeatSourceAndReactionIntensityUnits', 'LengthUnits', 'LoadingUnits', 'PressureUnits', 'SlopeUnits', 'SpeedUnits', 'SurfaceAreaToVolumeUnits', 'TemperatureUnits', 'TimeUnits']
class AreaUnits:
    class AreaUnitsEnum:
        """
        Members:
        
          SquareFeet
        
          Acres
        
          Hectares
        
          SquareMeters
        
          SquareMiles
        
          SquareKilometers
        """
        Acres: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.Acres: 1>
        Hectares: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.Hectares: 2>
        SquareFeet: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.SquareFeet: 0>
        SquareKilometers: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.SquareKilometers: 5>
        SquareMeters: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.SquareMeters: 3>
        SquareMiles: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.SquareMiles: 4>
        __members__: typing.ClassVar[dict[str, AreaUnits.AreaUnitsEnum]]  # value = {'SquareFeet': <AreaUnitsEnum.SquareFeet: 0>, 'Acres': <AreaUnitsEnum.Acres: 1>, 'Hectares': <AreaUnitsEnum.Hectares: 2>, 'SquareMeters': <AreaUnitsEnum.SquareMeters: 3>, 'SquareMiles': <AreaUnitsEnum.SquareMiles: 4>, 'SquareKilometers': <AreaUnitsEnum.SquareKilometers: 5>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    Acres: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.Acres: 1>
    Hectares: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.Hectares: 2>
    SquareFeet: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.SquareFeet: 0>
    SquareKilometers: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.SquareKilometers: 5>
    SquareMeters: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.SquareMeters: 3>
    SquareMiles: typing.ClassVar[AreaUnits.AreaUnitsEnum]  # value = <AreaUnitsEnum.SquareMiles: 4>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: AreaUnits.AreaUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: AreaUnits.AreaUnitsEnum) -> float:
        ...
class BasalAreaUnits:
    class BasalAreaUnitsEnum:
        """
        Members:
        
          SquareFeetPerAcre
        
          SquareMetersPerHectare
        """
        SquareFeetPerAcre: typing.ClassVar[BasalAreaUnits.BasalAreaUnitsEnum]  # value = <BasalAreaUnitsEnum.SquareFeetPerAcre: 0>
        SquareMetersPerHectare: typing.ClassVar[BasalAreaUnits.BasalAreaUnitsEnum]  # value = <BasalAreaUnitsEnum.SquareMetersPerHectare: 1>
        __members__: typing.ClassVar[dict[str, BasalAreaUnits.BasalAreaUnitsEnum]]  # value = {'SquareFeetPerAcre': <BasalAreaUnitsEnum.SquareFeetPerAcre: 0>, 'SquareMetersPerHectare': <BasalAreaUnitsEnum.SquareMetersPerHectare: 1>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    SquareFeetPerAcre: typing.ClassVar[BasalAreaUnits.BasalAreaUnitsEnum]  # value = <BasalAreaUnitsEnum.SquareFeetPerAcre: 0>
    SquareMetersPerHectare: typing.ClassVar[BasalAreaUnits.BasalAreaUnitsEnum]  # value = <BasalAreaUnitsEnum.SquareMetersPerHectare: 1>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: BasalAreaUnits.BasalAreaUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: BasalAreaUnits.BasalAreaUnitsEnum) -> float:
        ...
class DensityUnits:
    class DensityUnitsEnum:
        """
        Members:
        
          PoundsPerCubicFoot
        
          KilogramsPerCubicMeter
        """
        KilogramsPerCubicMeter: typing.ClassVar[DensityUnits.DensityUnitsEnum]  # value = <DensityUnitsEnum.KilogramsPerCubicMeter: 1>
        PoundsPerCubicFoot: typing.ClassVar[DensityUnits.DensityUnitsEnum]  # value = <DensityUnitsEnum.PoundsPerCubicFoot: 0>
        __members__: typing.ClassVar[dict[str, DensityUnits.DensityUnitsEnum]]  # value = {'PoundsPerCubicFoot': <DensityUnitsEnum.PoundsPerCubicFoot: 0>, 'KilogramsPerCubicMeter': <DensityUnitsEnum.KilogramsPerCubicMeter: 1>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    KilogramsPerCubicMeter: typing.ClassVar[DensityUnits.DensityUnitsEnum]  # value = <DensityUnitsEnum.KilogramsPerCubicMeter: 1>
    PoundsPerCubicFoot: typing.ClassVar[DensityUnits.DensityUnitsEnum]  # value = <DensityUnitsEnum.PoundsPerCubicFoot: 0>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: DensityUnits.DensityUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: DensityUnits.DensityUnitsEnum) -> float:
        ...
class FirelineIntensityUnits:
    class FirelineIntensityUnitsEnum:
        """
        Members:
        
          BtusPerFootPerSecond
        
          BtusPerFootPerMinute
        
          KilojoulesPerMeterPerSecond
        
          KilojoulesPerMeterPerMinute
        
          KilowattsPerMeter
        """
        BtusPerFootPerMinute: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.BtusPerFootPerMinute: 1>
        BtusPerFootPerSecond: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.BtusPerFootPerSecond: 0>
        KilojoulesPerMeterPerMinute: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.KilojoulesPerMeterPerMinute: 3>
        KilojoulesPerMeterPerSecond: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.KilojoulesPerMeterPerSecond: 2>
        KilowattsPerMeter: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.KilowattsPerMeter: 4>
        __members__: typing.ClassVar[dict[str, FirelineIntensityUnits.FirelineIntensityUnitsEnum]]  # value = {'BtusPerFootPerSecond': <FirelineIntensityUnitsEnum.BtusPerFootPerSecond: 0>, 'BtusPerFootPerMinute': <FirelineIntensityUnitsEnum.BtusPerFootPerMinute: 1>, 'KilojoulesPerMeterPerSecond': <FirelineIntensityUnitsEnum.KilojoulesPerMeterPerSecond: 2>, 'KilojoulesPerMeterPerMinute': <FirelineIntensityUnitsEnum.KilojoulesPerMeterPerMinute: 3>, 'KilowattsPerMeter': <FirelineIntensityUnitsEnum.KilowattsPerMeter: 4>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    BtusPerFootPerMinute: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.BtusPerFootPerMinute: 1>
    BtusPerFootPerSecond: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.BtusPerFootPerSecond: 0>
    KilojoulesPerMeterPerMinute: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.KilojoulesPerMeterPerMinute: 3>
    KilojoulesPerMeterPerSecond: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.KilojoulesPerMeterPerSecond: 2>
    KilowattsPerMeter: typing.ClassVar[FirelineIntensityUnits.FirelineIntensityUnitsEnum]  # value = <FirelineIntensityUnitsEnum.KilowattsPerMeter: 4>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: FirelineIntensityUnits.FirelineIntensityUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: FirelineIntensityUnits.FirelineIntensityUnitsEnum) -> float:
        ...
class FractionUnits:
    class FractionUnitsEnum:
        """
        Members:
        
          Fraction
        
          Percent
        """
        Fraction: typing.ClassVar[FractionUnits.FractionUnitsEnum]  # value = <FractionUnitsEnum.Fraction: 0>
        Percent: typing.ClassVar[FractionUnits.FractionUnitsEnum]  # value = <FractionUnitsEnum.Percent: 1>
        __members__: typing.ClassVar[dict[str, FractionUnits.FractionUnitsEnum]]  # value = {'Fraction': <FractionUnitsEnum.Fraction: 0>, 'Percent': <FractionUnitsEnum.Percent: 1>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    Fraction: typing.ClassVar[FractionUnits.FractionUnitsEnum]  # value = <FractionUnitsEnum.Fraction: 0>
    Percent: typing.ClassVar[FractionUnits.FractionUnitsEnum]  # value = <FractionUnitsEnum.Percent: 1>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: FractionUnits.FractionUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: FractionUnits.FractionUnitsEnum) -> float:
        ...
class HeatOfCombustionUnits:
    class HeatOfCombustionUnitsEnum:
        """
        Members:
        
          BtusPerPound
        
          KilojoulesPerKilogram
        """
        BtusPerPound: typing.ClassVar[HeatOfCombustionUnits.HeatOfCombustionUnitsEnum]  # value = <HeatOfCombustionUnitsEnum.BtusPerPound: 0>
        KilojoulesPerKilogram: typing.ClassVar[HeatOfCombustionUnits.HeatOfCombustionUnitsEnum]  # value = <HeatOfCombustionUnitsEnum.KilojoulesPerKilogram: 1>
        __members__: typing.ClassVar[dict[str, HeatOfCombustionUnits.HeatOfCombustionUnitsEnum]]  # value = {'BtusPerPound': <HeatOfCombustionUnitsEnum.BtusPerPound: 0>, 'KilojoulesPerKilogram': <HeatOfCombustionUnitsEnum.KilojoulesPerKilogram: 1>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    BtusPerPound: typing.ClassVar[HeatOfCombustionUnits.HeatOfCombustionUnitsEnum]  # value = <HeatOfCombustionUnitsEnum.BtusPerPound: 0>
    KilojoulesPerKilogram: typing.ClassVar[HeatOfCombustionUnits.HeatOfCombustionUnitsEnum]  # value = <HeatOfCombustionUnitsEnum.KilojoulesPerKilogram: 1>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: HeatOfCombustionUnits.HeatOfCombustionUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: HeatOfCombustionUnits.HeatOfCombustionUnitsEnum) -> float:
        ...
class HeatPerUnitAreaUnits:
    class HeatPerUnitAreaUnitsEnum:
        """
        Members:
        
          BtusPerSquareFoot
        
          KilojoulesPerSquareMeter
        
          KilowattSecondsPerSquareMeter
        """
        BtusPerSquareFoot: typing.ClassVar[HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum]  # value = <HeatPerUnitAreaUnitsEnum.BtusPerSquareFoot: 0>
        KilojoulesPerSquareMeter: typing.ClassVar[HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum]  # value = <HeatPerUnitAreaUnitsEnum.KilojoulesPerSquareMeter: 1>
        KilowattSecondsPerSquareMeter: typing.ClassVar[HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum]  # value = <HeatPerUnitAreaUnitsEnum.KilowattSecondsPerSquareMeter: 2>
        __members__: typing.ClassVar[dict[str, HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum]]  # value = {'BtusPerSquareFoot': <HeatPerUnitAreaUnitsEnum.BtusPerSquareFoot: 0>, 'KilojoulesPerSquareMeter': <HeatPerUnitAreaUnitsEnum.KilojoulesPerSquareMeter: 1>, 'KilowattSecondsPerSquareMeter': <HeatPerUnitAreaUnitsEnum.KilowattSecondsPerSquareMeter: 2>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    BtusPerSquareFoot: typing.ClassVar[HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum]  # value = <HeatPerUnitAreaUnitsEnum.BtusPerSquareFoot: 0>
    KilojoulesPerSquareMeter: typing.ClassVar[HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum]  # value = <HeatPerUnitAreaUnitsEnum.KilojoulesPerSquareMeter: 1>
    KilowattSecondsPerSquareMeter: typing.ClassVar[HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum]  # value = <HeatPerUnitAreaUnitsEnum.KilowattSecondsPerSquareMeter: 2>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: HeatPerUnitAreaUnits.HeatPerUnitAreaUnitsEnum) -> float:
        ...
class HeatSinkUnits:
    class HeatSinkUnitsEnum:
        """
        Members:
        
          BtusPerCubicFoot
        
          KilojoulesPerCubicMeter
        """
        BtusPerCubicFoot: typing.ClassVar[HeatSinkUnits.HeatSinkUnitsEnum]  # value = <HeatSinkUnitsEnum.BtusPerCubicFoot: 0>
        KilojoulesPerCubicMeter: typing.ClassVar[HeatSinkUnits.HeatSinkUnitsEnum]  # value = <HeatSinkUnitsEnum.KilojoulesPerCubicMeter: 1>
        __members__: typing.ClassVar[dict[str, HeatSinkUnits.HeatSinkUnitsEnum]]  # value = {'BtusPerCubicFoot': <HeatSinkUnitsEnum.BtusPerCubicFoot: 0>, 'KilojoulesPerCubicMeter': <HeatSinkUnitsEnum.KilojoulesPerCubicMeter: 1>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    BtusPerCubicFoot: typing.ClassVar[HeatSinkUnits.HeatSinkUnitsEnum]  # value = <HeatSinkUnitsEnum.BtusPerCubicFoot: 0>
    KilojoulesPerCubicMeter: typing.ClassVar[HeatSinkUnits.HeatSinkUnitsEnum]  # value = <HeatSinkUnitsEnum.KilojoulesPerCubicMeter: 1>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: HeatSinkUnits.HeatSinkUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: HeatSinkUnits.HeatSinkUnitsEnum) -> float:
        ...
class HeatSourceAndReactionIntensityUnits:
    class HeatSourceAndReactionIntensityUnitsEnum:
        """
        Members:
        
          BtusPerSquareFootPerMinute
        
          BtusPerSquareFootPerSecond
        
          KilojoulesPerSquareMeterPerSecond
        
          KilojoulesPerSquareMeterPerMinute
        
          KilowattsPerSquareMeter
        """
        BtusPerSquareFootPerMinute: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.BtusPerSquareFootPerMinute: 0>
        BtusPerSquareFootPerSecond: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.BtusPerSquareFootPerSecond: 1>
        KilojoulesPerSquareMeterPerMinute: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.KilojoulesPerSquareMeterPerMinute: 3>
        KilojoulesPerSquareMeterPerSecond: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.KilojoulesPerSquareMeterPerSecond: 2>
        KilowattsPerSquareMeter: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.KilowattsPerSquareMeter: 4>
        __members__: typing.ClassVar[dict[str, HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]]  # value = {'BtusPerSquareFootPerMinute': <HeatSourceAndReactionIntensityUnitsEnum.BtusPerSquareFootPerMinute: 0>, 'BtusPerSquareFootPerSecond': <HeatSourceAndReactionIntensityUnitsEnum.BtusPerSquareFootPerSecond: 1>, 'KilojoulesPerSquareMeterPerSecond': <HeatSourceAndReactionIntensityUnitsEnum.KilojoulesPerSquareMeterPerSecond: 2>, 'KilojoulesPerSquareMeterPerMinute': <HeatSourceAndReactionIntensityUnitsEnum.KilojoulesPerSquareMeterPerMinute: 3>, 'KilowattsPerSquareMeter': <HeatSourceAndReactionIntensityUnitsEnum.KilowattsPerSquareMeter: 4>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    BtusPerSquareFootPerMinute: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.BtusPerSquareFootPerMinute: 0>
    BtusPerSquareFootPerSecond: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.BtusPerSquareFootPerSecond: 1>
    KilojoulesPerSquareMeterPerMinute: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.KilojoulesPerSquareMeterPerMinute: 3>
    KilojoulesPerSquareMeterPerSecond: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.KilojoulesPerSquareMeterPerSecond: 2>
    KilowattsPerSquareMeter: typing.ClassVar[HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum]  # value = <HeatSourceAndReactionIntensityUnitsEnum.KilowattsPerSquareMeter: 4>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: HeatSourceAndReactionIntensityUnits.HeatSourceAndReactionIntensityUnitsEnum) -> float:
        ...
class LengthUnits:
    class LengthUnitsEnum:
        """
        Members:
        
          Feet
        
          Inches
        
          Millimeters
        
          Centimeters
        
          Meters
        
          Chains
        
          Miles
        
          Kilometers
        """
        Centimeters: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Centimeters: 3>
        Chains: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Chains: 5>
        Feet: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Feet: 0>
        Inches: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Inches: 1>
        Kilometers: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Kilometers: 7>
        Meters: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Meters: 4>
        Miles: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Miles: 6>
        Millimeters: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Millimeters: 2>
        __members__: typing.ClassVar[dict[str, LengthUnits.LengthUnitsEnum]]  # value = {'Feet': <LengthUnitsEnum.Feet: 0>, 'Inches': <LengthUnitsEnum.Inches: 1>, 'Millimeters': <LengthUnitsEnum.Millimeters: 2>, 'Centimeters': <LengthUnitsEnum.Centimeters: 3>, 'Meters': <LengthUnitsEnum.Meters: 4>, 'Chains': <LengthUnitsEnum.Chains: 5>, 'Miles': <LengthUnitsEnum.Miles: 6>, 'Kilometers': <LengthUnitsEnum.Kilometers: 7>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    Centimeters: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Centimeters: 3>
    Chains: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Chains: 5>
    Feet: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Feet: 0>
    Inches: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Inches: 1>
    Kilometers: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Kilometers: 7>
    Meters: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Meters: 4>
    Miles: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Miles: 6>
    Millimeters: typing.ClassVar[LengthUnits.LengthUnitsEnum]  # value = <LengthUnitsEnum.Millimeters: 2>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: LengthUnits.LengthUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: LengthUnits.LengthUnitsEnum) -> float:
        ...
class LoadingUnits:
    class LoadingUnitsEnum:
        """
        Members:
        
          PoundsPerSquareFoot
        
          TonsPerAcre
        
          TonnesPerHectare
        
          KilogramsPerSquareMeter
        """
        KilogramsPerSquareMeter: typing.ClassVar[LoadingUnits.LoadingUnitsEnum]  # value = <LoadingUnitsEnum.KilogramsPerSquareMeter: 3>
        PoundsPerSquareFoot: typing.ClassVar[LoadingUnits.LoadingUnitsEnum]  # value = <LoadingUnitsEnum.PoundsPerSquareFoot: 0>
        TonnesPerHectare: typing.ClassVar[LoadingUnits.LoadingUnitsEnum]  # value = <LoadingUnitsEnum.TonnesPerHectare: 2>
        TonsPerAcre: typing.ClassVar[LoadingUnits.LoadingUnitsEnum]  # value = <LoadingUnitsEnum.TonsPerAcre: 1>
        __members__: typing.ClassVar[dict[str, LoadingUnits.LoadingUnitsEnum]]  # value = {'PoundsPerSquareFoot': <LoadingUnitsEnum.PoundsPerSquareFoot: 0>, 'TonsPerAcre': <LoadingUnitsEnum.TonsPerAcre: 1>, 'TonnesPerHectare': <LoadingUnitsEnum.TonnesPerHectare: 2>, 'KilogramsPerSquareMeter': <LoadingUnitsEnum.KilogramsPerSquareMeter: 3>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    KilogramsPerSquareMeter: typing.ClassVar[LoadingUnits.LoadingUnitsEnum]  # value = <LoadingUnitsEnum.KilogramsPerSquareMeter: 3>
    PoundsPerSquareFoot: typing.ClassVar[LoadingUnits.LoadingUnitsEnum]  # value = <LoadingUnitsEnum.PoundsPerSquareFoot: 0>
    TonnesPerHectare: typing.ClassVar[LoadingUnits.LoadingUnitsEnum]  # value = <LoadingUnitsEnum.TonnesPerHectare: 2>
    TonsPerAcre: typing.ClassVar[LoadingUnits.LoadingUnitsEnum]  # value = <LoadingUnitsEnum.TonsPerAcre: 1>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: LoadingUnits.LoadingUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: LoadingUnits.LoadingUnitsEnum) -> float:
        ...
class PressureUnits:
    class PressureUnitsEnum:
        """
        Members:
        
          Pascal
        
          HectoPascal
        
          KiloPascal
        
          MegaPascal
        
          GigaPascal
        
          Bar
        
          Atmosphere
        
          TechnicalAtmosphere
        
          PoundPerSquareInch
        """
        Atmosphere: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.Atmosphere: 6>
        Bar: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.Bar: 5>
        GigaPascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.GigaPascal: 4>
        HectoPascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.HectoPascal: 1>
        KiloPascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.KiloPascal: 2>
        MegaPascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.MegaPascal: 3>
        Pascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.Pascal: 0>
        PoundPerSquareInch: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.PoundPerSquareInch: 8>
        TechnicalAtmosphere: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.TechnicalAtmosphere: 7>
        __members__: typing.ClassVar[dict[str, PressureUnits.PressureUnitsEnum]]  # value = {'Pascal': <PressureUnitsEnum.Pascal: 0>, 'HectoPascal': <PressureUnitsEnum.HectoPascal: 1>, 'KiloPascal': <PressureUnitsEnum.KiloPascal: 2>, 'MegaPascal': <PressureUnitsEnum.MegaPascal: 3>, 'GigaPascal': <PressureUnitsEnum.GigaPascal: 4>, 'Bar': <PressureUnitsEnum.Bar: 5>, 'Atmosphere': <PressureUnitsEnum.Atmosphere: 6>, 'TechnicalAtmosphere': <PressureUnitsEnum.TechnicalAtmosphere: 7>, 'PoundPerSquareInch': <PressureUnitsEnum.PoundPerSquareInch: 8>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    Atmosphere: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.Atmosphere: 6>
    Bar: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.Bar: 5>
    GigaPascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.GigaPascal: 4>
    HectoPascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.HectoPascal: 1>
    KiloPascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.KiloPascal: 2>
    MegaPascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.MegaPascal: 3>
    Pascal: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.Pascal: 0>
    PoundPerSquareInch: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.PoundPerSquareInch: 8>
    TechnicalAtmosphere: typing.ClassVar[PressureUnits.PressureUnitsEnum]  # value = <PressureUnitsEnum.TechnicalAtmosphere: 7>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: PressureUnits.PressureUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: PressureUnits.PressureUnitsEnum) -> float:
        ...
class SlopeUnits:
    class SlopeUnitsEnum:
        """
        Members:
        
          Degrees
        
          Percent
        """
        Degrees: typing.ClassVar[SlopeUnits.SlopeUnitsEnum]  # value = <SlopeUnitsEnum.Degrees: 0>
        Percent: typing.ClassVar[SlopeUnits.SlopeUnitsEnum]  # value = <SlopeUnitsEnum.Percent: 1>
        __members__: typing.ClassVar[dict[str, SlopeUnits.SlopeUnitsEnum]]  # value = {'Degrees': <SlopeUnitsEnum.Degrees: 0>, 'Percent': <SlopeUnitsEnum.Percent: 1>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    Degrees: typing.ClassVar[SlopeUnits.SlopeUnitsEnum]  # value = <SlopeUnitsEnum.Degrees: 0>
    Percent: typing.ClassVar[SlopeUnits.SlopeUnitsEnum]  # value = <SlopeUnitsEnum.Percent: 1>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: SlopeUnits.SlopeUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: SlopeUnits.SlopeUnitsEnum) -> float:
        ...
class SpeedUnits:
    class SpeedUnitsEnum:
        """
        Members:
        
          FeetPerMinute
        
          ChainsPerHour
        
          MetersPerSecond
        
          MetersPerMinute
        
          MetersPerHour
        
          MilesPerHour
        
          KilometersPerHour
        """
        ChainsPerHour: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.ChainsPerHour: 1>
        FeetPerMinute: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.FeetPerMinute: 0>
        KilometersPerHour: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.KilometersPerHour: 6>
        MetersPerHour: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.MetersPerHour: 4>
        MetersPerMinute: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.MetersPerMinute: 3>
        MetersPerSecond: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.MetersPerSecond: 2>
        MilesPerHour: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.MilesPerHour: 5>
        __members__: typing.ClassVar[dict[str, SpeedUnits.SpeedUnitsEnum]]  # value = {'FeetPerMinute': <SpeedUnitsEnum.FeetPerMinute: 0>, 'ChainsPerHour': <SpeedUnitsEnum.ChainsPerHour: 1>, 'MetersPerSecond': <SpeedUnitsEnum.MetersPerSecond: 2>, 'MetersPerMinute': <SpeedUnitsEnum.MetersPerMinute: 3>, 'MetersPerHour': <SpeedUnitsEnum.MetersPerHour: 4>, 'MilesPerHour': <SpeedUnitsEnum.MilesPerHour: 5>, 'KilometersPerHour': <SpeedUnitsEnum.KilometersPerHour: 6>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    ChainsPerHour: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.ChainsPerHour: 1>
    FeetPerMinute: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.FeetPerMinute: 0>
    KilometersPerHour: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.KilometersPerHour: 6>
    MetersPerHour: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.MetersPerHour: 4>
    MetersPerMinute: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.MetersPerMinute: 3>
    MetersPerSecond: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.MetersPerSecond: 2>
    MilesPerHour: typing.ClassVar[SpeedUnits.SpeedUnitsEnum]  # value = <SpeedUnitsEnum.MilesPerHour: 5>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: SpeedUnits.SpeedUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: SpeedUnits.SpeedUnitsEnum) -> float:
        ...
class SurfaceAreaToVolumeUnits:
    class SurfaceAreaToVolumeUnitsEnum:
        """
        Members:
        
          SquareFeetOverCubicFeet
        
          SquareMetersOverCubicMeters
        
          SquareInchesOverCubicInches
        
          SquareCentimetersOverCubicCentimeters
        """
        SquareCentimetersOverCubicCentimeters: typing.ClassVar[SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]  # value = <SurfaceAreaToVolumeUnitsEnum.SquareCentimetersOverCubicCentimeters: 3>
        SquareFeetOverCubicFeet: typing.ClassVar[SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]  # value = <SurfaceAreaToVolumeUnitsEnum.SquareFeetOverCubicFeet: 0>
        SquareInchesOverCubicInches: typing.ClassVar[SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]  # value = <SurfaceAreaToVolumeUnitsEnum.SquareInchesOverCubicInches: 2>
        SquareMetersOverCubicMeters: typing.ClassVar[SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]  # value = <SurfaceAreaToVolumeUnitsEnum.SquareMetersOverCubicMeters: 1>
        __members__: typing.ClassVar[dict[str, SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]]  # value = {'SquareFeetOverCubicFeet': <SurfaceAreaToVolumeUnitsEnum.SquareFeetOverCubicFeet: 0>, 'SquareMetersOverCubicMeters': <SurfaceAreaToVolumeUnitsEnum.SquareMetersOverCubicMeters: 1>, 'SquareInchesOverCubicInches': <SurfaceAreaToVolumeUnitsEnum.SquareInchesOverCubicInches: 2>, 'SquareCentimetersOverCubicCentimeters': <SurfaceAreaToVolumeUnitsEnum.SquareCentimetersOverCubicCentimeters: 3>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    SquareCentimetersOverCubicCentimeters: typing.ClassVar[SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]  # value = <SurfaceAreaToVolumeUnitsEnum.SquareCentimetersOverCubicCentimeters: 3>
    SquareFeetOverCubicFeet: typing.ClassVar[SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]  # value = <SurfaceAreaToVolumeUnitsEnum.SquareFeetOverCubicFeet: 0>
    SquareInchesOverCubicInches: typing.ClassVar[SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]  # value = <SurfaceAreaToVolumeUnitsEnum.SquareInchesOverCubicInches: 2>
    SquareMetersOverCubicMeters: typing.ClassVar[SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum]  # value = <SurfaceAreaToVolumeUnitsEnum.SquareMetersOverCubicMeters: 1>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: SurfaceAreaToVolumeUnits.SurfaceAreaToVolumeUnitsEnum) -> float:
        ...
class TemperatureUnits:
    class TemperatureUnitsEnum:
        """
        Members:
        
          Fahrenheit
        
          Celsius
        
          Kelvin
        """
        Celsius: typing.ClassVar[TemperatureUnits.TemperatureUnitsEnum]  # value = <TemperatureUnitsEnum.Celsius: 1>
        Fahrenheit: typing.ClassVar[TemperatureUnits.TemperatureUnitsEnum]  # value = <TemperatureUnitsEnum.Fahrenheit: 0>
        Kelvin: typing.ClassVar[TemperatureUnits.TemperatureUnitsEnum]  # value = <TemperatureUnitsEnum.Kelvin: 2>
        __members__: typing.ClassVar[dict[str, TemperatureUnits.TemperatureUnitsEnum]]  # value = {'Fahrenheit': <TemperatureUnitsEnum.Fahrenheit: 0>, 'Celsius': <TemperatureUnitsEnum.Celsius: 1>, 'Kelvin': <TemperatureUnitsEnum.Kelvin: 2>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    Celsius: typing.ClassVar[TemperatureUnits.TemperatureUnitsEnum]  # value = <TemperatureUnitsEnum.Celsius: 1>
    Fahrenheit: typing.ClassVar[TemperatureUnits.TemperatureUnitsEnum]  # value = <TemperatureUnitsEnum.Fahrenheit: 0>
    Kelvin: typing.ClassVar[TemperatureUnits.TemperatureUnitsEnum]  # value = <TemperatureUnitsEnum.Kelvin: 2>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: TemperatureUnits.TemperatureUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: TemperatureUnits.TemperatureUnitsEnum) -> float:
        ...
class TimeUnits:
    class TimeUnitsEnum:
        """
        Members:
        
          Minutes
        
          Seconds
        
          Hours
        
          Days
        
          Years
        """
        Days: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Days: 3>
        Hours: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Hours: 2>
        Minutes: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Minutes: 0>
        Seconds: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Seconds: 1>
        Years: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Years: 4>
        __members__: typing.ClassVar[dict[str, TimeUnits.TimeUnitsEnum]]  # value = {'Minutes': <TimeUnitsEnum.Minutes: 0>, 'Seconds': <TimeUnitsEnum.Seconds: 1>, 'Hours': <TimeUnitsEnum.Hours: 2>, 'Days': <TimeUnitsEnum.Days: 3>, 'Years': <TimeUnitsEnum.Years: 4>}
        def __eq__(self, other: typing.Any) -> bool:
            ...
        def __getstate__(self) -> int:
            ...
        def __hash__(self) -> int:
            ...
        def __index__(self) -> int:
            ...
        def __init__(self, value: int) -> None:
            ...
        def __int__(self) -> int:
            ...
        def __ne__(self, other: typing.Any) -> bool:
            ...
        def __repr__(self) -> str:
            ...
        def __setstate__(self, state: int) -> None:
            ...
        def __str__(self) -> str:
            ...
        @property
        def name(self) -> str:
            ...
        @property
        def value(self) -> int:
            ...
    Days: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Days: 3>
    Hours: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Hours: 2>
    Minutes: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Minutes: 0>
    Seconds: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Seconds: 1>
    Years: typing.ClassVar[TimeUnits.TimeUnitsEnum]  # value = <TimeUnitsEnum.Years: 4>
    @staticmethod
    def fromBaseUnits(arg0: float, arg1: TimeUnits.TimeUnitsEnum) -> float:
        ...
    @staticmethod
    def toBaseUnits(arg0: float, arg1: TimeUnits.TimeUnitsEnum) -> float:
        ...



from pyrothermel import behave_core, units, modes
from typing import Union

_FM_INDEX = 255

class PyrothermelRun:
    def __init__(self, fuel_model: 'FuelModel', moisture_scenario: 'MoistureScenario', wind_speed: float,
                 units_preset: Union[str, 'UnitsPreset'] = 'metric', wind_input_mode: str = 'direct_midflame',
                 wind_direction: float = 0, slope: float = 0., aspect: float = 0,
                 air_temperature: Union[float, None] = None, canopy_base_height: Union[float, None] = None,
                 canopy_bulk_density: Union[float, None] = None, canopy_cover: Union[float, None] = None,
                 canopy_height: Union[float, None] = None, canopy_ratio: Union[float, None] = None) -> None:

        """
        Initializes a PyrothermelRun object for fire behavior simulation.

        Args:
            fuel_model (FuelModel): FuelModel object containing fuel loading values
            moisture_scenario (MoistureScenario): MoistureScenario object containing fuel moisture values
            wind_speed (float): Wind speed value. Default metric unit: km/hr, default US unit: mi/hr.
            units_preset (Union[str, UnitsPreset], optional): Can be 'metric', 'us_standard', or a custom UnitsPreset object. Sets numerous input/output measurement units.
            wind_input_mode (str): Input height mode for wind. Options are 'direct_midflame', 'ten_meter', or 'twenty_foot'. If 'ten_meter' or 'twenty_foot' is used, must provide canopy height, canopy cover, canopy ratio to perform automatic adjustments. 
            wind_direction (float): Direction of wind in degrees relative to North.
            slope (float): Terrain slope. Default unit is degrees.
            aspect (float): Terrain aspect in degrees relative to North.
            air_temperature (float): Ambient air temperature, used in scorch model. If None, defaults to 30 degrees C or 85 degrees F.
            canopy_base_height (float): Required for crown fire simulations. Default unit is meters for metric or feet for US. 
            canopy_bulk_density (float, optional): Required for crown fire simulations. Default unit is kg/m^3 for metric or lb/ft^3 for US.
            canopy_cover (float): Fraction of canopy cover. Only required if wind_input_mode != 'direct_midflame'. Default unit is fraction between 0 and 1.
            canopy_height (float): Total height of canopy. Only required if wind_input_mode != 'direct_midflame'. Default unit is meters for metric or feet for US.
            canopy_ratio (float): Ratio of total height where significant foliage is present. Only required if wind_input_mode != 'direct_midflame'. Default unit is fraction between 0 and 1.
        """

        # Instantiate objects required by behave_core
        self._fuel_models = behave_core.FuelModels()
        self._mortality_species_table = behave_core.SpeciesMasterTable()

        # Instantiate objects where default value is None
        self.canopy_base_height = None
        self.canopy_bulk_density = None
        self.canopy_cover = None
        self.canopy_height = None
        self.canopy_ratio = None
        self.units = _get_units_preset(units_preset)
        if air_temperature is None:
            if self.units.temperature_units == units.TemperatureUnits.Celsius:
                air_temperature = 30.
            elif self.units.temperature_units == units.TemperatureUnits.Fahrenheit:
                air_temperature = 85.
            elif self.units.temperature_units == units.TemperatureUnits.Kelvin:
                air_temperature = 302.

        # Set values using user-provided and/or default values
        self.update_inputs(fuel_model=fuel_model, moisture_scenario=moisture_scenario, wind_speed=wind_speed, units_preset=units_preset, wind_input_mode=wind_input_mode, wind_direction=wind_direction, slope=slope, aspect=aspect, air_temperature=air_temperature,canopy_base_height=canopy_base_height,canopy_bulk_density=canopy_bulk_density, canopy_cover=canopy_cover, canopy_height=canopy_height, canopy_ratio=canopy_ratio)

    def update_inputs(self,fuel_model=None, moisture_scenario=None, wind_speed=None, units_preset=None, wind_input_mode=None, wind_direction=None, slope=None, aspect=None, air_temperature=None, canopy_base_height=None, canopy_bulk_density=None, canopy_cover=None, canopy_height=None, canopy_ratio=None):

        """
        Update one or more input values.

        Args:
            fuel_model (FuelModel): FuelModel object containing fuel loading values
            moisture_scenario (MoistureScenario): MoistureScenario object containing fuel moisture values
            wind_speed (float): Wind speed value. Default metric unit: km/hr, default US unit: mi/hr.
            units_preset (Union[str, UnitsPreset], optional): Can be 'metric', 'us_standard', or a custom UnitsPreset object. Sets numerous input/output measurement units.
            wind_input_mode (str): Input height mode for wind. Options are 'direct_midflame', 'ten_meter', or 'twenty_foot'. If 'ten_meter' or 'twenty_foot' is used, must provide canopy height, canopy cover, canopy ratio to perform automatic adjustments.
            wind_direction (float): Direction of wind in degrees relative to North.
            slope (float): Terrain slope. Default unit is degrees.
            aspect (float): Terrain aspect in degrees relative to North.
            air_temperature (float): Ambient air temperature, used in scorch model. If None, defaults to 30 degrees C or 85 degrees F.
            canopy_base_height (float): Required for crown fire simulations. Default unit is meters for metric or feet for US.
            canopy_bulk_density (float, optional): Required for crown fire simulations. Default unit is kg/m^3 for metric or lb/ft^3 for US.
            canopy_cover (float): Fraction of canopy cover. Only required if wind_input_mode != 'direct_midflame'. Default unit is fraction between 0 and 1.
            canopy_height (float): Total height of canopy. Only required if wind_input_mode != 'direct_midflame'. Default unit is meters for metric or feet for US.
            canopy_ratio (float): Ratio of total height where significant foliage is present. Only required if wind_input_mode != 'direct_midflame'. Default unit is fraction between 0 and 1.
        """

        if units_preset is not None:
            self.units = _get_units_preset(units_preset)
        if moisture_scenario is not None:
            self.moisture_scenario = moisture_scenario
        if wind_speed is not None:
            self.wind_speed = wind_speed
        if wind_input_mode is not None:
            self.wind_input_mode = wind_input_mode
        if wind_direction is not None:
            self.wind_direction = wind_direction
        if slope is not None:
            if slope>=0:
                self.slope = slope
            else:
                raise ValueError('Slope must be positive')
            #TODO: Add more validation
        if aspect is not None:
            self.aspect = aspect
        if air_temperature is not None:
            self.air_temperature = air_temperature
        if canopy_bulk_density is not None:
            self.canopy_bulk_density = canopy_bulk_density
        if canopy_base_height is not None:
            self.canopy_base_height = canopy_base_height
        if canopy_cover is not None:
            self.canopy_cover = canopy_cover
        if canopy_height is not None:
            self.canopy_height = canopy_height
        if canopy_ratio is not None:
            self.canopy_ratio = canopy_ratio


        self.wind_input_mode, canopy_cover, canopy_height, canopy_ratio = self._validate_wind_input_mode(self.wind_input_mode,
                                                                                                    self.canopy_cover,
                                                                                                    self.canopy_height,
                                                                                                    self.canopy_ratio)


        self._update_fuel_model(fuel_model)

        self._run = behave_core.BehaveRun(self._fuel_models, self._mortality_species_table)

        self._run.surface.updateSurfaceInputs(_FM_INDEX,
                                              self.moisture_scenario.moisture_one_hour,
                                              self.moisture_scenario.moisture_ten_hour,
                                              self.moisture_scenario.moisture_hundred_hour,
                                              self.moisture_scenario.moisture_live_herbaceous,
                                              self.moisture_scenario.moisture_live_woody,
                                              self.moisture_scenario.units,
                                              self.wind_speed,
                                              self.units.windspeed_units,
                                              self.wind_input_mode,
                                              self.wind_direction,
                                              modes.WindAndSpreadOrientationMode.RelativeToNorth,
                                              self.slope,
                                              self.units.slope_units,
                                              self.aspect,
                                              canopy_cover,
                                              self.units.fraction_units,
                                              canopy_height,
                                              self.units.length_units,
                                              canopy_ratio,
                                              self.units.fraction_units)


    def _update_fuel_model(self,fuel_model=None):
        if fuel_model is not None:
            self.fuel_model = fuel_model
        self._fuel_models.setCustomFuelModel(_FM_INDEX,
                                             self.fuel_model.code,
                                             self.fuel_model.name,
                                             self.fuel_model.fuel_bed_depth,
                                             self.fuel_model.units.length_units,
                                             self.fuel_model.moisture_of_extinction_dead,
                                             self.fuel_model.units.fraction_units,
                                             self.fuel_model.heat_of_combustion_dead,
                                             self.fuel_model.heat_of_combustion_live,
                                             self.fuel_model.units.heat_of_combustion_units,
                                             self.fuel_model.fuel_load_one_hour,
                                             self.fuel_model.fuel_load_ten_hour,
                                             self.fuel_model.fuel_load_hundred_hour,
                                             self.fuel_model.fuel_load_live_herbaceous,
                                             self.fuel_model.fuel_load_live_woody,
                                             self.fuel_model.units.loading_units,
                                             self.fuel_model.savr_one_hour,
                                             self.fuel_model.savr_live_herbaceous,
                                             self.fuel_model.savr_live_woody,
                                             self.fuel_model.units.savr_units,
                                             self.fuel_model.is_dynamic)

    def _validate_wind_input_mode(self,wind_input_mode,canopy_cover,canopy_height,canopy_ratio):
        wind_input_mode = _validate_wind_input_mode(wind_input_mode, canopy_cover, canopy_height, canopy_ratio)

        if self.canopy_cover is None:
            canopy_cover = 0
        else:
            canopy_cover = self.canopy_cover

        if self.canopy_height is None:
            canopy_height = 0
        else:
            canopy_height = self.canopy_height

        if self.canopy_ratio is None:
            canopy_ratio = 0
        else:
            canopy_ratio = self.canopy_ratio

        return [wind_input_mode,canopy_cover,canopy_height,canopy_ratio]

    def run_surface_fire_in_direction_of_max_spread(self)->dict:
        """
        Runs SURFACE module from behave, adds results to PyrothermelRun object as properties and returns dictionary with results.

        Args:
        None

        Returns:
        spread_rate: surface fire rate of spread. Default unit is km/hr for metric, chains/hr for US
        flame_length: surface fire flame length. Default unit is m for metric, feet for US.
        fireline_intensity: Byram's fireline intensity. Default unit is kW/m for metric, btus/ft/s for US.
        scorch_height: Canopy scorch height from surface fire. Default unit is m for metric, feet for US.
        midflame_windspeed: Adjusted midflame windspeed. Default unit is km/hr for metric, mi/hr for US.
        direction: Direction of maximum spread relative to North


        :return: dict
        """

        self.update_inputs()
        self._run.surface.doSurfaceRunInDirectionOfMaxSpread()
        self.spread_rate = self._run.surface.getSpreadRate(self.units.spread_rate_units)
        self.flame_length = self._run.surface.getFlameLength(self.units.length_units)
        self.fireline_intensity = self._run.surface.getFirelineIntensity(self.units.fireline_intensity_units)
        self.midflame_windspeed = self._run.surface.getMidflameWindspeed(self.units.windspeed_units)
        self.direction_of_max_spread = self._run.surface.getDirectionOfMaxSpread()
        self.scorch_height = self._run.mortality.calculateScorchHeight(self.fireline_intensity, self.units.fireline_intensity_units,
                                                                      self.midflame_windspeed, self.units.windspeed_units, self.air_temperature,
                                                                      self.units.temperature_units, self.units.length_units)
        results = {'spread_rate': self.spread_rate,'flame_length':self.flame_length,
                   'fireline_intensity':self.fireline_intensity,'scorch_height':self.scorch_height,
                   'midflame_windspeed':self.midflame_windspeed,'direction':self.direction_of_max_spread}
        return results

    def run_surface_fire_in_direction_of_interest(self, direction):
        """
        Runs SURFACE module from behave, adds results to PyrothermelRun object as properties and returns dictionary with results.
        Calculates results for direction of interest relative to point fire center.

        Args:
        None

        Returns:
        spread_rate: surface fire rate of spread. Default unit is km/hr for metric, chains/hr for US
        flame_length: surface fire flame length. Default unit is m for metric, feet for US.
        fireline_intensity: Byram's fireline intensity. Default unit is kW/m for metric, btus/ft/s for US.
        midflame_windspeed: Adjusted midflame windspeed. Default unit is km/hr for metric, mi/hr for US.
        scorch_height: Canopy scorch height from surface fire. Default unit is m for metric, feet for US.
        direction: Direction of interest specified by user.

        :return: dict

        Example:

        run = PyrothermelRun(FuelModel.from_existing('TL8'), MoistureScenario.from_existing(1,2), 30)
        results_max_spread = run.run_surface_fire_in_direction_of_max_spread()
        dir_max_spread = results_max_spread['direction']
        results_flanking1 = run.run_surface_fire_in_direction_of_interest(dir_max_spread - 90)
        results_flanking2 = run.run_surface_fire_in_direction_of_interest(dir_max_spread - 270)
        results_backing = run.run_surface_fire_in_direction_of_interest(dir_max_spread - 180)
        """

        self.update_inputs()
        self._run.surface.doSurfaceRunInDirectionOfInterest(direction,modes.SurfaceFireSpreadDirectionMode.FromIgnitionPoint)
        self.spread_rate = self._run.surface.getSpreadRate(self.units.spread_rate_units)
        self.flame_length = self._run.surface.getFlameLength(self.units.length_units)
        self.fireline_intensity = self._run.surface.getFirelineIntensity(self.units.fireline_intensity_units)
        self.midflame_windspeed = self._run.surface.getMidflameWindspeed(self.units.windspeed_units)
        self.direction_of_max_spread = self._run.surface.getDirectionOfMaxSpread()
        self.scorch_height = self._run.mortality.calculateScorchHeight(self.fireline_intensity, self.units.fireline_intensity_units,
                                                                      self.midflame_windspeed, self.units.windspeed_units, self.air_temperature,
                                                                      self.units.temperature_units, self.units.length_units)
        results = {'spread_rate': self.spread_rate,'flame_length':self.flame_length,
                   'fireline_intensity':self.fireline_intensity,'scorch_height':self.scorch_height,
                   'midflame_windspeed':self.midflame_windspeed,'direction':self.direction_of_max_spread}
        return results

    def run_crown_fire_scott_and_reinhardt(self):
        """
        Runs CROWN module from behave using Scott and Reinhardt approach. Adds results to PyrothermelRun object as
        properties and returns dictionary with results.

        Must run run_surface_fire_in_direction_of_max_spread before running this function.
        Must set canopy_base_height and canopy_bulk_density model inputs.

        Args:
        None

        Returns:
        spread_rate: Combined surface/crown fire rate of spread. Default unit is km/hr for metric, chains/hr for US
        flame_length:  Combined surface/crown fire flame length. Default unit is m for metric, feet for US.
        fireline_intensity:  Combined surface/crown Byram's fireline intensity. Default unit is kW/m for metric, btus/ft/s for US.
        scorch_height: Canopy scorch height from surface fire. Default unit is m for metric, feet for US.
        transition_ratio: Current wind speed / winds peed to initiate a crown fire. If transition_ratio >= 1, fire type changes.
        active_ratio: Current wind speed / winds peed to propegate a crown fire. If active_ratio >= 1, fire type changes.
        fire_type: 'surface': transition_ratio < 1, active_ratio < 1; 'crowning': transition_ratio >= 1, active ratio >= 1; 'torching': transition_ratio >= 1, active_ratio < 1; 'conditional_crown_fire': transition_ratio < 1, active_ratio >= 1
        midflame_windspeed: Adjusted midflame windspeed. Default unit is km/hr for metric, mi/hr for US.
        direction: Direction of maximum spread relative to North


        :return: dict
        """
        if self.canopy_base_height is None or self.canopy_bulk_density is None:
            raise ValueError('canopy_base_height and canopy_bulk_density must be provided')

        wind_input_mode, canopy_cover, canopy_height, canopy_ratio = self._validate_wind_input_mode(self.wind_input_mode, self.canopy_cover, self.canopy_height, self.canopy_ratio)

        self._run.crown.updateCrownInputs(_FM_INDEX, self.moisture_scenario.moisture_one_hour,
                                           self.moisture_scenario.moisture_ten_hour,
                                           self.moisture_scenario.moisture_hundred_hour,
                                           self.moisture_scenario.moisture_live_herbaceous,
                                           self.moisture_scenario.moisture_live_woody,
                                           self.moisture_scenario.foliar_moisture, self.moisture_scenario.units,
                                           self.wind_speed, self.units.windspeed_units,
                                           self.wind_input_mode, self.wind_direction, modes.WindAndSpreadOrientationMode.RelativeToNorth,
                                           self.slope, self.units.slope_units, self.aspect, canopy_cover,
                                           self.units.fraction_units, canopy_height, self.canopy_base_height,
                                           self.units.length_units, canopy_ratio, self.units.fraction_units,
                                           self.canopy_bulk_density, self.units.density_units
                                        )
        self._run.crown.doCrownRunScottAndReinhardt()

        self.spread_rate = self._run.crown.getFinalSpreadRate(self.units.spread_rate_units)
        self.fireline_intensity = self._run.crown.getFinalFirelineIntensity(self.units.fireline_intensity_units)
        self.flame_length = self._run.crown.getFinalFlameLength(self.units.length_units)
        self.transition_ratio = self._run.crown.getTransitionRatio()
        self.active_ratio = self._run.crown.getActiveRatio()
        self.fire_type = self._run.crown.getFireType()

        results = {'spread_rate': self.spread_rate,'flame_length':self.flame_length,
                   'fireline_intensity':self.fireline_intensity, 'scorch_height':self.scorch_height,
                   'transition_ratio':self.transition_ratio, 'active_ratio':self.active_ratio,'fire_type':self.fire_type,
                   'midflame_windspeed':self.midflame_windspeed,'direction':self.direction_of_max_spread
                   }
        return results

    def run_crown_fire_rothermel(self):
        """
        Runs CROWN module from behave using Scott and Reinhardt approach. Adds results to PyrothermelRun object as
        properties and returns dictionary with results.

        Must run run_surface_fire_in_direction_of_max_spread before running this function.
        Must set canopy_base_height and canopy_bulk_density model inputs.

        Args:
        None

        Returns:
        spread_rate: Combined surface/crown fire rate of spread. Default unit is km/hr for metric, chains/hr for US
        flame_length:  Combined surface/crown fire flame length. Default unit is m for metric, feet for US.
        fireline_intensity:  Combined surface/crown Byram's fireline intensity. Default unit is kW/m for metric, btus/ft/s for US.
        scorch_height: Canopy scorch height from surface fire. Default unit is m for metric, feet for US.
        transition_ratio: Current wind speed / winds peed to initiate a crown fire. If transition_ratio >= 1, fire type changes.
        active_ratio: Current wind speed / winds peed to propegate a crown fire. If active_ratio >= 1, fire type changes.
        fire_type: 'surface': transition_ratio < 1, active_ratio < 1; 'crowning': transition_ratio >= 1, active ratio >= 1; 'torching': transition_ratio >= 1, active_ratio < 1; 'conditional_crown_fire': transition_ratio < 1, active_ratio >= 1
        midflame_windspeed: Adjusted midflame windspeed. Default unit is km/hr for metric, mi/hr for US.
        direction: Direction of maximum spread relative to North


        :return: dict
        """

        if self.canopy_base_height is None or self.canopy_bulk_density is None:
            raise ValueError('canopy_base_height and canopy_bulk_density must be provided')

        wind_input_mode, canopy_cover, canopy_height, canopy_ratio = self._validate_wind_input_mode(
            self.wind_input_mode, self.canopy_cover, self.canopy_height, self.canopy_ratio)

        self._run.crown.updateCrownInputs(_FM_INDEX, self.moisture_scenario.moisture_one_hour,
                                          self.moisture_scenario.moisture_ten_hour,
                                          self.moisture_scenario.moisture_hundred_hour,
                                          self.moisture_scenario.moisture_live_herbaceous,
                                          self.moisture_scenario.moisture_live_woody,
                                          self.moisture_scenario.foliar_moisture, self.moisture_scenario.units,
                                          self.wind_speed, self.units.windspeed_units,
                                          self.wind_input_mode, self.wind_direction,
                                          modes.WindAndSpreadOrientationMode.RelativeToNorth,
                                          self.slope, self.units.slope_units, self.aspect, canopy_cover,
                                          self.units.fraction_units, canopy_height, self.canopy_base_height,
                                          self.units.length_units, canopy_ratio, self.units.fraction_units,
                                          self.canopy_bulk_density, self.units.density_units
                                          )
        self._run.crown.doCrownRunScottAndReinhardt()

        self.spread_rate = self._run.crown.getFinalSpreadRate(self.units.spread_rate_units)
        self.fireline_intensity = self._run.crown.getFinalFirelineIntensity(self.units.fireline_intensity_units)
        self.flame_length = self._run.crown.getCrownFinalFlameLength(self.units.length_units)
        self.transition_ratio = self._run.crown.getTransitionRatio()
        self.active_ratio = self._run.crown.getActiveRatio()
        self.fire_type = self._run.crown.getFireType()

        results = {'spread_rate': self.spread_rate, 'flame_length': self.flame_length,
                   'fireline_intensity': self.fireline_intensity, 'scorch_height': self.scorch_height,
                   'transition_ratio': self.transition_ratio, 'active_ratio': self.active_ratio,
                   'fire_type': self.fire_type, 'midflame_windspeed':self.midflame_windspeed,'direction':self.direction_of_max_spread}
        return results

    def calculate_torching_index(self, max_wind_speed=99, step=1):
        """
        Calculates the minimum wind speed required to initiate a crown fire under the stored fuel and moisture conditions.
        Calculated via an iterative process of increasing wind speed.

        Args:
        max_wind_speed: maximum wind speed to be considered. Default units is km/hr for metric or mi/hr for US.
        step: step size used for increasing wind speed in each iteration.

        :return: float
        """
        for wind_speed in range(0, max_wind_speed, step):
            self.update_inputs(wind_speed=wind_speed)
            self.run_surface_fire_in_direction_of_max_spread()
            results = self.run_crown_fire_scott_and_reinhardt()
            if results['transition_ratio']>=1:
                return wind_speed
        return wind_speed

    def calculate_crowning_index(self, max_wind_speed=99, step=1):
        """
        Calculates the minimum wind speed required to propegate a crown fire under the stored fuel and moisture conditions.
        Calculated via an iterative process of increasing wind speed.

        Args:
        max_wind_speed: maximum wind speed to be considered. Default units is km/hr for metric or mi/hr for US.
        step: step size used for increasing wind speed in each iteration.

        :return: float
        """
        for wind_speed in range(0, max_wind_speed, step):
            self.update_inputs(wind_speed=wind_speed)
            self.run_surface_fire_in_direction_of_max_spread()
            results = self.run_crown_fire_scott_and_reinhardt()
            if results['active_ratio']>=1:
                return wind_speed
        return wind_speed

class UnitsPreset:
    def __init__(self, area_units, basal_area_units, length_units, loading_units, savr_units, pressure_units, spread_rate_units,
                 windspeed_units, fraction_units, slope_units, density_units, heat_of_combustion_units,
                 heat_sink_units, heat_per_unit_area_units, heat_source_and_reaction_intensity_units,
                 fireline_intensity_units, temperature_units, time_units):
        """
        UnitsPreset object is used to determine the measurement units for input and output values in simulations. Use
        UnitsPreset.metric() or UnitsPreset.us_standard() to get default presets. Modify these presets or create your own
        using objects from the pyrothermel.units module which stores bindings to c++ enums for all unit types supported
        in Behave.

        Example:
            from pyrothermel import UnitsPreset, units
            my_units = UnitsPreset.metric()
            my_units.windspeed_units = units.SpeedUnits.MetersPerSecond
        """

        self.area_units = area_units
        self.basal_area_units = basal_area_units
        self.length_units = length_units
        self.loading_units = loading_units
        self.savr_units = savr_units
        self.pressure_units = pressure_units
        self.spread_rate_units = spread_rate_units
        self.windspeed_units = windspeed_units
        self.fraction_units = fraction_units
        self.slope_units = slope_units
        self.density_units = density_units
        self.heat_of_combustion_units = heat_of_combustion_units
        self.heat_sink_units = heat_sink_units
        self.heat_per_unit_area_units = heat_per_unit_area_units
        self.heat_source_and_reaction_intensity_units = heat_source_and_reaction_intensity_units
        self.fireline_intensity_units = fireline_intensity_units
        self.temperature_units = temperature_units
        self.time_units = time_units

    @classmethod
    def metric(cls):
        """
        units.AreaUnits.SquareMeters, units.BasalAreaUnits.SquareMetersPerHectare, units.LengthUnits.Meters,
         units.LoadingUnits.KilogramsPerSquareMeter, units.SurfaceAreaToVolumeUnits.SquareMetersOverCubicMeters,
         units.PressureUnits.KiloPascal, units.SpeedUnits.KilometersPerHour, units.SpeedUnits.KilometersPerHour,
         units.FractionUnits.Fraction, units.SlopeUnits.Degrees, units.DensityUnits.KilogramsPerCubicMeter,
         units.HeatOfCombustionUnits.KilojoulesPerKilogram, units.HeatSinkUnits.KilojoulesPerCubicMeter,
         units.HeatPerUnitAreaUnits.KilojoulesPerSquareMeter, units.HeatSourceAndReactionIntensityUnits.KilowattsPerSquareMeter,
         units.FirelineIntensityUnits.KilowattsPerMeter, units.TemperatureUnits.Celsius, units.TimeUnits.Hours
        """
        preset = cls(units.AreaUnits.SquareMeters, units.BasalAreaUnits.SquareMetersPerHectare, units.LengthUnits.Meters,
                     units.LoadingUnits.KilogramsPerSquareMeter, units.SurfaceAreaToVolumeUnits.SquareMetersOverCubicMeters,
                     units.PressureUnits.KiloPascal, units.SpeedUnits.KilometersPerHour, units.SpeedUnits.KilometersPerHour,
                     units.FractionUnits.Fraction, units.SlopeUnits.Degrees, units.DensityUnits.KilogramsPerCubicMeter,
                     units.HeatOfCombustionUnits.KilojoulesPerKilogram, units.HeatSinkUnits.KilojoulesPerCubicMeter,
                     units.HeatPerUnitAreaUnits.KilojoulesPerSquareMeter, units.HeatSourceAndReactionIntensityUnits.KilowattsPerSquareMeter,
                     units.FirelineIntensityUnits.KilowattsPerMeter, units.TemperatureUnits.Celsius, units.TimeUnits.Hours)
        return preset

    @classmethod
    def us_standard(cls):
        """
        units.AreaUnits.Acres, units.BasalAreaUnits.SquareFeetPerAcre, units.LengthUnits.Feet,
         units.LoadingUnits.TonsPerAcre, units.SurfaceAreaToVolumeUnits.SquareFeetOverCubicFeet,
         units.PressureUnits.PoundPerSquareInch, units.SpeedUnits.ChainsPerHour, units.SpeedUnits.MilesPerHour,
         units.FractionUnits.Fraction, units.SlopeUnits.Degrees, units.DensityUnits.PoundsPerCubicFoot,
         units.HeatOfCombustionUnits.BtusPerPound, units.HeatSinkUnits.BtusPerCubicFoot,
         units.HeatPerUnitAreaUnits.BtusPerSquareFoot, units.HeatSourceAndReactionIntensityUnits.BtusPerSquareFootPerSecond,
         units.FirelineIntensityUnits.BtusPerFootPerSecond, units.TemperatureUnits.Fahrenheit, units.TimeUnits.Hours
        :return:
        """
        preset = cls(units.AreaUnits.Acres, units.BasalAreaUnits.SquareFeetPerAcre, units.LengthUnits.Feet,
                     units.LoadingUnits.TonsPerAcre, units.SurfaceAreaToVolumeUnits.SquareFeetOverCubicFeet,
                     units.PressureUnits.PoundPerSquareInch, units.SpeedUnits.ChainsPerHour, units.SpeedUnits.MilesPerHour,
                     units.FractionUnits.Fraction, units.SlopeUnits.Degrees, units.DensityUnits.PoundsPerCubicFoot,
                     units.HeatOfCombustionUnits.BtusPerPound, units.HeatSinkUnits.BtusPerCubicFoot,
                     units.HeatPerUnitAreaUnits.BtusPerSquareFoot, units.HeatSourceAndReactionIntensityUnits.BtusPerSquareFootPerSecond,
                     units.FirelineIntensityUnits.BtusPerFootPerSecond, units.TemperatureUnits.Fahrenheit, units.TimeUnits.Hours)
        return preset

def _get_units_preset(units_preset) -> 'UnitsPreset':
    if type(units_preset) is UnitsPreset:
       return units_preset
    elif units_preset == 'metric':
        return UnitsPreset.metric()
    elif units_preset == 'us_standard':
        return UnitsPreset.us_standard()
    else:
        raise ValueError("units_preset must be 'metric' or 'us_standard' or UnitsPreset object")

def _validate_wind_input_mode(wind_input_mode, canopy_cover, canopy_height, canopy_ratio):
    if type(wind_input_mode) is not modes.WindHeightInputMode:
        if wind_input_mode == 'direct_midflame':
            wind_input_mode = modes.WindHeightInputMode.DirectMidflame
        elif wind_input_mode == 'ten_meter':
            wind_input_mode = modes.WindHeightInputMode.TenMeter
        elif wind_input_mode == 'twenty_foot':
            wind_input_mode = modes.WindHeightInputMode.TwentyFoot
        else:
            raise ValueError("wind_input_mode must be 'direct_midflame', 'ten_meter', or 'twenty_foot'")

    if wind_input_mode != modes.WindHeightInputMode.DirectMidflame:
        if canopy_ratio is None or canopy_height is None or canopy_cover is None:
            raise ValueError(
                "Canopy ratio, canopy height, and canopy cover must be provided if wind_input_mode is not 'direct_midflame'")

    return wind_input_mode

class FuelModel:
    def __init__(self,
                 units_preset,
                fuel_model_number: int,
                code: str,
                name: str,
                fuel_bed_depth: float,
                moisture_of_extinction_dead: float,
                heat_of_combustion_dead: float,
                heat_of_combustion_live: float,
                fuel_load_one_hour: float,
                fuel_load_ten_hour: float,
                fuel_load_hundred_hour: float,
                fuel_load_live_herbaceous: float,
                fuel_load_live_woody: float,
                savr_one_hour: float,
                savr_live_herbaceous: float,
                savr_live_woody: float,
                is_dynamic: bool,
    ) -> None:
        """
        Initiates FuelModel object with the provided parameters. It is recommended to use the FuelModel.from_existing()
        method to get values from an existing fuel model, then you can modify the values as needed for a custom model.

        Args:
            units_preset: can be 'metric', 'us_standard', or a UnitsPreset object
            fuel_model_number (int): The fuel model number. Value is stored for reference but not used in modelling.
            code (str): The code representing the fuel model. Value is stored for reference but not used in modelling.
            name (str): The name of the fuel model. Value is stored for reference but not used in modelling.
            fuel_bed_depth (float): Depth of the fuel bed. Default units are meters for metric, feet for US.
            moisture_of_extinction_dead (float): Moisture of extinction for dead fuels. Default unit is fraction between 0 and 1.
            heat_of_combustion_dead (float): Heat of combustion for dead fuels. Default units are kJ/kg for metric or btu/lb for US.
            heat_of_combustion_live (float): Heat of combustion for live fuels. Default units are kJ/kg for metric or btu/lb for US.
            fuel_load_one_hour (float): Fuel load for one-hour interval. Default units are kg/m^2 for metric or tons/ac for US.
            fuel_load_ten_hour (float): Fuel load for ten-hour interval. Default units are kg/m^2 for metric or tons/ac for US.
            fuel_load_hundred_hour (float): Fuel load for hundred-hour interval. Default units are kg/m^2 for metric or tons/ac for US.
            fuel_load_live_herbaceous (float): Live herbaceous fuel load. Default units are kg/m^2 for metric or tons/ac for US.
            fuel_load_live_woody (float): Live woody fuel load. Default units are kg/m^2 for metric or tons/ac for US.
            savr_one_hour (float): Surface area-to-volume ratio for one-hour fuels. Default units are m^2/m^3 for metric or ft^2/ft^3 for US.
            savr_live_herbaceous (float): Surface area-to-volume ratio for live herbaceous fuels. Default units are m^2/m^3 for metric or ft^2/ft^3 for US.
            savr_live_woody (float): Surface area-to-volume ratio for live woody fuels. Default units are m^2/m^3 for metric or ft^2/ft^3 for US.
            is_dynamic (bool): Indicates if the model is dynamic.
        """
        self.units = _get_units_preset(units_preset)
        self.fuel_model_number = fuel_model_number
        self.code = code
        self.name = name
        self.fuel_bed_depth = fuel_bed_depth
        self.moisture_of_extinction_dead = moisture_of_extinction_dead
        self.heat_of_combustion_dead = heat_of_combustion_dead
        self.heat_of_combustion_live = heat_of_combustion_live
        self.fuel_load_one_hour = fuel_load_one_hour
        self.fuel_load_ten_hour = fuel_load_ten_hour
        self.fuel_load_hundred_hour = fuel_load_hundred_hour
        self.fuel_load_live_herbaceous = fuel_load_live_herbaceous
        self.fuel_load_live_woody = fuel_load_live_woody
        self.savr_one_hour = savr_one_hour
        self.savr_live_herbaceous = savr_live_herbaceous
        self.savr_live_woody = savr_live_woody
        self.is_dynamic = is_dynamic

    @classmethod
    def from_existing(cls,identifier:Union[str,int],units_preset='metric')->'FuelModel':
        """
        Initiates FuelModel object from an existing standard fuel model (Scott and Burgan 2005).
        Default fuel loading units are kg/m^2 for metric or tons/ac for US.

        Args:
        identifier: int representing index ID of standard fuel model or str representing model code, e.g. 'TL8'
        units_preset: can be 'metric', 'us_standard', or a UnitsPreset object.

        """
        units_preset = _get_units_preset(units_preset)
        fuelModels = behave_core.FuelModels()
        if type(identifier) is not int:
            for i in range(220):
                code = fuelModels.getFuelCode(i)
                if (identifier == code):
                    identifier = i
                    break
            if i == 219:
                raise ValueError("Could not find model matching this identifier")

        mod = cls(units_preset, identifier,
                  fuelModels.getFuelCode(identifier),
                  fuelModels.getFuelName(identifier),
                  fuelModels.getFuelbedDepth(identifier,units_preset.length_units),
                  fuelModels.getMoistureOfExtinctionDead(identifier,units_preset.fraction_units),
                  fuelModels.getHeatOfCombustionDead(identifier,units_preset.heat_of_combustion_units),
                  fuelModels.getHeatOfCombustionLive(identifier,units_preset.heat_of_combustion_units),
                  fuelModels.getFuelLoadOneHour(identifier,units_preset.loading_units),
                  fuelModels.getFuelLoadTenHour(identifier,units_preset.loading_units),
                  fuelModels.getFuelLoadHundredHour(identifier,units_preset.loading_units),
                  fuelModels.getFuelLoadLiveHerbaceous(identifier,units_preset.loading_units),
                  fuelModels.getFuelLoadLiveWoody(identifier,units_preset.loading_units),
                  fuelModels.getSavrOneHour(identifier,units_preset.savr_units),
                  fuelModels.getSavrLiveHerbaceous(identifier,units_preset.savr_units),
                  fuelModels.getSavrLiveWoody(identifier,units_preset.savr_units),
                  fuelModels.getIsDynamic(identifier))

        return mod

    @property
    def bulk_density(self):
        bd = (self.fuel_load_one_hour + self.fuel_load_ten_hour + self.fuel_load_hundred_hour +
              self.fuel_load_live_herbaceous + self.fuel_load_live_woody) / self.fuel_bed_depth
        return bd

    # @property
    # def packing_ratio(self):
    #     particle_density_constant = 32 #standard particle density is 32 lbs/ft^3
    #     if self.units.loading_units == units.LoadingUnits.PoundsPerSquareFoot and self.units.length_units == units.LengthUnits.Feet:
    #         return self.bulk_density / particle_density_constant
    #     elif self.units.loading_units == units.LoadingUnits.KilogramsPerSquareMeter and self.units.length_units == units.LengthUnits.Meter:
    #         particle_density_constant *= 16.01846337396 #conversion from lbs/ft^3 to kg/m^3
    #         return self.bulk_density / particle_density_constant
    #     else:
    #         raise ValueError("Packing ratio currently supports loading_units kg/m^2 with length_units m or loading_units lbs/ft^2 with length_units feet")

    @bulk_density.setter
    def bulk_density(self, bulk_density):
        """Adjusts fuel bed depth such that bulk density matches specified value"""
        self.fuel_bed_depth = (self.fuel_load_one_hour + self.fuel_load_ten_hour + self.fuel_load_hundred_hour +
                               self.fuel_load_live_herbaceous + self.fuel_load_live_woody) / bulk_density



class MoistureScenario:
    """
    Initiates MoistureScenario from provided inputs. Default values are fractions between 0 and 1.
    """
    def __init__(self, moisture_one_hour:float, moisture_ten_hour:float, moisture_hundred_hour:float, moisture_live_herbaceous:float, moisture_live_woody:float, foliar_moisture = 1, fraction_units='fraction'):

        if type(fraction_units) != units.FractionUnits.FractionUnitsEnum:
            if fraction_units == 'fraction':
                fraction_units = units.FractionUnits.Fraction
            elif fraction_units == 'percent':
                fraction_units = units.FractionUnits.Percent
            else:
                raise ValueError("units must be 'fraction' or 'percent'")

        if fraction_units == units.FractionUnits.Fraction:
            if moisture_one_hour > 3:
                raise ValueError("Expected moisture as fraction between 0 and 1")
            if moisture_ten_hour > 3:
                raise ValueError("Expected moisture as fraction between 0 and 1")
            if moisture_hundred_hour > 3:
                raise ValueError("Expected moisture as fraction between 0 and 1")
            if moisture_live_woody > 3:
                raise ValueError("Expected moisture as fraction between 0 and 1")
            if moisture_live_herbaceous > 3:
                raise ValueError("Expected moisture as fraction between 0 and 1")
            if foliar_moisture > 3:
                raise ValueError("Expected moisture as fraction between 0 and 1")

        self.moisture_one_hour = moisture_one_hour
        self.moisture_ten_hour = moisture_ten_hour
        self.moisture_hundred_hour = moisture_hundred_hour
        self.moisture_live_herbaceous = moisture_live_herbaceous
        self.moisture_live_woody = moisture_live_woody
        self.foliar_moisture = foliar_moisture
        self.units = fraction_units

    @classmethod
    def from_existing(
            cls,
            dead_fuel_moisture_class: Union[int, str],
            live_fuel_moisture_class: Union[int, str],
            foliar_moisture: float = 1,
            fraction_units: str = 'fraction'
    ) -> 'MoistureScenario':
        """
            Create MoistureScenario from a preset found in Scott & Burgan (2005).

            Args:
            dead_fuel_moisture_class : int or str
                Dead fuel moisture scenario. Acceptable values are:
                - 1 or 'very_low'
                - 2 or 'low'
                - 3 or 'moderate'
                - 4 or 'high'

            live_fuel_moisture_class : int or str
                Live fuel moisture scenario. Acceptable values are:
                - 1 or 'very_low'
                - 2 or 'low'
                - 3 or 'moderate'
                - 4 or 'high'

            foliar_moisture : float
                The foliar moisture content, default is 1.

            fraction_units : str
                The units for fraction values. Can be 'fraction' or 'percent', but fraction is recommended.

            Returns:
            --------
            instance : MoistureScenario

            Raises:
            -------
            ValueError
                If either `dead_fuel_moisture_class` or `live_fuel_moisture_class` is not one of
                the accepted values (1, 2, 3, 4 or 'very_low', 'low', 'moderate', 'high').

            Fuel Moisture Settings:
            -----------------------
            Preset values for the given classes are:

            For `dead_fuel_moisture_class`:
            - 'very_low' or 1:
                - `one_hr` = 0.03
                - `ten_hr` = 0.04
                - `hundred_hr` = 0.05
            - 'low' or 2:
                - `one_hr` = 0.06
                - `ten_hr` = 0.07
                - `hundred_hr` = 0.08
            - 'moderate' or 3:
                - `one_hr` = 0.09
                - `ten_hr` = 0.1
                - `hundred_hr` = 0.11
            - 'high' or 4:
                - `one_hr` = 0.12
                - `ten_hr` = 0.13
                - `hundred_hr` = 0.14

            For `live_fuel_moisture_class`:
            - 'very_low' or 1:
                - `live_herb` = 0.3
                - `live_woody` = 0.6
            - 'low' or 2:
                - `live_herb` = 0.6
                - `live_woody` = 0.9
            - 'moderate' or 3:
                - `live_herb` = 0.9
                - `live_woody` = 1.2
            - 'high' or 4:
                - `live_herb` = 1.2
                - `live_woody` = 1.5

            Example Usage:
            --------------
            >>> instance = MyClass.from_existing(1, 2)
            >>> instance = MyClass.from_existing('very_low', 'low')

            """

        if dead_fuel_moisture_class == 1 or dead_fuel_moisture_class=='very_low':
            one_hr = .03
            ten_hr = .04
            hundred_hr = .05
        elif dead_fuel_moisture_class == 2 or dead_fuel_moisture_class=='low':
            one_hr = .06
            ten_hr = .07
            hundred_hr = .08
        elif dead_fuel_moisture_class == 3 or dead_fuel_moisture_class=='moderate':
            one_hr = .09
            ten_hr = .1
            hundred_hr = .11
        elif dead_fuel_moisture_class == 4 or dead_fuel_moisture_class=='high':
            one_hr = .12
            ten_hr = .13
            hundred_hr = .14
        else:
            raise ValueError('dead_fuel_moisture_class must one of 1:very_low, 2:low, 3:moderate, 4:high')

        if live_fuel_moisture_class == 1 or live_fuel_moisture_class=='very_low':
            live_herb = .3
            live_woody = .6
        elif live_fuel_moisture_class == 2 or live_fuel_moisture_class=='low':
            live_herb = .6
            live_woody = .9
        elif live_fuel_moisture_class == 3 or live_fuel_moisture_class=='moderate':
            live_herb = .9
            live_woody = 1.2
        elif live_fuel_moisture_class == 4 or live_fuel_moisture_class=='high':
            live_herb = 1.2
            live_woody = 1.5
        else:
            raise ValueError('live_fuel_moisture_class must one of 1:very_low, 2:low, 3:moderate, 4:high')

        return cls(one_hr, ten_hr, hundred_hr, live_herb, live_woody, foliar_moisture, fraction_units)

def calculate_characteristic_savr(savr_arr, loading_arr, particle_density):
    """Calculate characteristic surface-area-to-volume ratio for a fuel bed with mixed particle sizes.

    If both live and dead fuels are included, arrays must be 2-D with rows representing status classes and cols
    representing size classes. Particle density is usually taken as 512.6 kg/m^3 (32 lbs/ft^3).

    References SAVR values for cylinders

    | Diameter (cm) | SAVR (cm⁻¹) |
    | ------------- | ---------- |
    | 0.0356        | 114.83     |
    | 0.0610        | 65.61      |
    | 0.1016        | 39.37      |
    | 0.1524        | 24.61      |
    | 0.6350        | 6.30       |
    | 1.1176        | 3.58       | * std value for 10-hr fuel
    | 1.2700        | 3.15       |
    | 2.5400        | 1.57       |
    | 4.0640        | 0.98       | * std value for 100-hr fuel
    | 7.6200        | 0.52       |


    | Diameter (inches) | SAVR (feet⁻¹) |
    | ----------------- | ------------ |
    | 0.014             | 3,500        |
    | 0.024             | 2,000        | * typical for leaf litter
    | 0.040             | 1,200        |
    | 0.060             | 750          |
    | 0.250             | 192          |
    | 0.440             | 109          | * std value for 10-hr fuel
    | 0.500             | 96           |
    | 1.000             | 48           |
    | 1.600             | 30           | * std value for 100-hr fuel
    | 3.000             | 16           |


    """
    import numpy as np
    savr_arr = np.array(savr_arr).T
    loading_arr = np.array(loading_arr).T
    particle_density_arr = np.array(particle_density).T
    if particle_density_arr.shape == 1:
        particle_density_arr = np.ones_like(savr_arr) * particle_density_arr

    a_weights = savr_arr * loading_arr / particle_density_arr
    f_weights = a_weights / a_weights.sum(0)

    if len(savr_arr.shape) == 1:
        return np.sum(f_weights * savr_arr)

    elif len(savr_arr.shape)==2:
        f_weights_class = a_weights.sum(0) / a_weights.sum()
        savr_class = np.sum(f_weights * savr_arr, 0)
        return np.sum(f_weights_class * savr_class)
    else:
        raise ValueError('input arrays must have 1 or 2 dimensions')

def calculate_characteristic_load(savr_arr, loading_arr, particle_density, mineral_fraction=0.0555, units='m'):
    """Calculate characteristic fuel load for a fuel bed with mixed particle sizes.

    If both live and dead fuels are included, arrays must be 2-D with rows representing status classes and cols
    representing size classes."""

    import numpy as np
    savr_arr = np.array(savr_arr).T
    loading_arr = np.array(loading_arr).T
    particle_density_arr = np.array(particle_density).T
    mineral_fraction_arr = np.array(mineral_fraction).T
    if particle_density_arr.shape == 1:
        particle_density_arr = np.ones_like(savr_arr,float) * particle_density_arr

    if mineral_fraction_arr.shape == 1:
        mineral_fraction_arr = np.ones_like(savr_arr) * mineral_fraction_arr

    a_weights = savr_arr * loading_arr / particle_density_arr
    f_weights = a_weights / a_weights.sum(0)

    savr_classes = np.array([0,16,48,96,192,1200,np.inf])
    if units == 'm':
        savr_classes *= 3.2808
    elif units == 'cm':
        savr_classes /= 30.48
    elif units == 'ft':
        pass
    else:
        raise ValueError('units must either be "m", "cm", or "ft"')

    g_weights = np.zeros_like(savr_arr,float)
    if len(savr_arr.shape) == 1:
        for low, high in zip(savr_classes[:-1],savr_classes[1:]):
            mask = (savr_arr >= low) & (savr_arr <= high)
            g_weights[mask] = np.sum(f_weights[mask])
    elif len(savr_arr.shape) == 2:
        for low, high in zip(savr_classes[:-1],savr_classes[1:]):
            mask0 = (savr_arr[:,0] >= low) & (savr_arr[:,0] <= high)
            g_weights[mask0,0] = np.sum(f_weights[mask0,0])
            mask1 = (savr_arr[:,1] >= low) & (savr_arr[:,1] <= high)
            g_weights[mask1,1] = np.sum(f_weights[mask1,1])
    else:
        raise ValueError('input arrays must have 1 or 2 dimensions')


    load_net = loading_arr * (1 - mineral_fraction_arr)
    characteristic_mineral_fraction = np.sum(f_weights * mineral_fraction_arr, 0)

    load_net_class = np.sum(g_weights * load_net, 0)
    characteristic_load = load_net_class / (1-characteristic_mineral_fraction)
    return characteristic_load






#%%
import pyrothermel
import pandas as pd
import seaborn as sns
#%% md
# ## Setup Base Fuel Model and Moisture Scenario
#%%
moisture = pyrothermel.MoistureScenario.from_existing(dead_fuel_moisture_class='low',live_fuel_moisture_class='moderate')
fuel = pyrothermel.FuelModel.from_existing(identifier='TL8')
canopy_base_height = 2.5 # default unit is m
canopy_bulk_density = .1 # default unit is kg/m^3

# Print some fuel loading values from fuel model TL8
print([fuel.fuel_load_one_hour, fuel.fuel_load_ten_hour, fuel.fuel_load_hundred_hour])
print(fuel.units.loading_units)

#%% md
# ## Run Rothermel Models Across Range of Wind Speeds
#%%
results_list = []
for wind_speed in range(0,60,2):
    run = pyrothermel.PyrothermelRun(fuel,moisture,wind_speed,wind_input_mode='ten_meter',canopy_base_height=canopy_base_height,canopy_bulk_density=canopy_bulk_density,canopy_cover=.5,canopy_height=20,canopy_ratio=.6)
    results_surface = run.run_surface_fire_in_direction_of_max_spread()
    results_final = run.run_crown_fire_scott_and_reinhardt()
    results_final['wind_speed'] = wind_speed
    results_final['treatment'] = 'untreated'
    results_list.append(results_final)
    
untreated_crowning_index = run.calculate_crowning_index()
untreated_torching_index = run.calculate_torching_index()
    
df = pd.DataFrame(results_list)
df

#%% md
# ## Modify Fuel Loading and Recalculate
#%%
fuel.fuel_load_one_hour *= .5
fuel.fuel_load_ten_hour *= .5
fuel.fuel_load_hundred_hour *= .75
fuel.fuel_bed_depth *= .5

results_list = []
for wind_speed in range(0,60,2):
    run = pyrothermel.PyrothermelRun(fuel,moisture,wind_speed,wind_input_mode='ten_meter',canopy_base_height=2.5,canopy_bulk_density=.1,canopy_cover=.5,canopy_height=20,canopy_ratio=.6)
    results_surface = run.run_surface_fire_in_direction_of_max_spread()
    results_final = run.run_crown_fire_scott_and_reinhardt()
    results_final['wind_speed'] = wind_speed
    results_final['treatment'] = 'treated'
    results_list.append(results_final)

treated_crowning_index = run.calculate_crowning_index()
treated_torching_index = run.calculate_torching_index()

df_treated = pd.DataFrame(results_list)
df = pd.concat([df,df_treated])
#%% md
# ## Display Results
#%%
sns.lineplot(df,x='wind_speed',y='flame_length',hue='treatment').set(xlabel='Wind Speed (km/hr)',ylabel='Flame Length (m)')
#%%
print('Wind Speed to initiate crown fire in untreated stand: ', untreated_torching_index, ' km/hr')
print('Wind Speed to initiate crown fire in treated stand: ', treated_torching_index, ' km/hr')
print('Wind Speed to propagate crown fire in untreated stand: ', untreated_crowning_index, ' km/hr')
print('Wind Speed to propagate crown fire in treated stand: ', treated_crowning_index, ' km/hr')
#%%
